#!/usr/bin/perl -I /pyramid/sahoo/scripts/

package MyNetwork;

use Network;

sub new {
  my $proto = shift;
  my $class = ref($proto) || $proto;
  my %params = @_;
  my $self  = {};
  $self->{'file'}  = undef;
  $self->{'type'}  = undef;
  $self->{'ortholog'}  = undef;
  $self->{'annotation'}  = undef;
  $self->{'idxFile'}  = undef;

  $self->{'network'}  = undef;
  $self->{'balanced'}  = undef;
  $self->{'balancedhash'}  = undef;
  $self->{'namehash'}  = undef;
  $self->{'orgHash'}  = undef;
  $self->{'list_i'}  = [];
  $self->{'list_j'} = [];
  $self->{'i_saved'} = undef;
  $self->{'j_saved'} = undef;
  $self->{'relationCache'} = {};
  $self->{'cachesize'} = 1000;

  foreach (keys(%params)) {
    $self->{$_} = $params{$_};
  }

  bless ($self, $class);
  return $self;
}

sub init {
  my $self = shift;
  $self->{'network'} = Network->new(file => $self->{'file'}, mode => "r");
  $self->{'network'}->readNetworkFile(); # read headers
  $self->{'balanced'} = $self->{'network'}->getBalanced();
  $self->{'balancedhash'} = {};
  for(my $i = 0; $i < scalar(@{$self->{'balanced'}}); $i++) {
    $self->{'balancedhash'}->{$self->{'balanced'}->[$i]} = $i;
  }
  if (defined $self->{'annotation'} && $self->{'annotation'} =~ /.idx$/) {
    $self->{'nameHash'} = Network::getNameHashFromIdx($self->{'annotation'});
  } 
  else {
    $self->{'nameHash'} = Network::getNameHashFromAnnotation($self->{'annotation'});
  }
  $self->{'orgHash'} = Network::getOrthologHash($self->{'ortholog'}, $self->{'type'});
}

sub populateOrthologs {
  my ($self, $type) = @_;
  $self->{'orgHash'} = Network::getOrthologHash($self->{'ortholog'}, $type);
}

sub populateOrthologFromNetwork {
  my ($self, $mn) = @_;
  if (!defined $self->{'orgHash'}) {
    $self->{'orgHash'} = {};
  }
  my $nhash = $mn->{'nameHash'};
  my $hash = {};
  foreach my $id (keys %{$nhash}) {
    foreach my $n (split("///", $nhash->{$id})) {
      $n =~ s/\s//g;
      next if ($n =~ /^\s*$/);
      next if ($n eq "---");
      next if ($n =~ /Unknown/i);
      push @{$hash->{uc($n)}}, $id;
    }
  }
  my $nhash = $self->{'nameHash'};
  foreach my $id (keys %{$nhash}) {
    foreach my $n (split("///", $nhash->{$id})) {
      $n =~ s/\s//g;
      if (defined $hash->{uc($n)}) {
        my $h = {};
        foreach (@{$self->{'orgHash'}->{$id}}) {
          $h->{$_} = 1;
        }
        foreach (@{$hash->{uc($n)}}) {
          $h->{$_} = 1;
        }
        $self->{'orgHash'}->{$id} = [keys %{$h}];
      }
    }
  }
}

sub getAffyIds {
  my ($self, $file) = @_;
  my $hash = Network::getIDHash($file);
  my $res = [];
  if (defined $self->{'annotation'} && $self->{'annotation'} =~ /.idx$/) {
    my $nhash = $self->{'nameHash'};
    my $h = {};
    foreach my $id (keys %{$nhash}) {
      foreach my $n (split("///", $nhash->{$id})) {
        $n =~ s/\s//g;
        if (defined $hash->{$n}) {
          $h->{$id} = 1;
        }
      }
    }
    @$res = keys %{$h};
  } 
  else {
    $res = Network::unigeneToAffy($self->{'annotation'}, $hash);
  }
  return $res;
}

sub getIDs {
  my ($self, $file) = @_;
  my $hash = Network::getIDHash($file);
  my $listAffy = [keys %{$hash}];
  return $listAffy;
}

sub getIndex {
  my ($self, $pi) = @_;
  return $self->{'balancedhash'}->{$pi};
}

sub isBalanced {
  my ($self, $pi) = @_;
  return defined $self->{'balancedhash'}->{$pi};
}

sub pIndex {
  my ($self, $i) = @_;
  return $self->{'balanced'}->[$i];
}

sub getNumOrthologs {
  my ($self, $pi) = @_;
  my $orgHash = $self->{'orgHash'};
  return scalar(@{$orgHash->{$pi}});
}

sub getOrthologs {
  my ($self, $pi) = @_;
  my $orgHash = $self->{'orgHash'};
  return $orgHash->{$pi};
}

sub getOrthologsFromArray {
  my ($self, $pis) = @_;
  my $orgHash = $self->{'orgHash'};
  my $res = {};
  foreach my $i (@$pis) {
    foreach my $j (@{$orgHash->{$i}}) {
      $res->{$j} = 1;
    }
  }
  return [keys %$res];
}

sub setList_i {
  my ($self, $pi) = @_;
  my $balancedhash = $self->{'balancedhash'};
  my $orgHash = $self->{'orgHash'};
  my @hlist_i = map { $balancedhash->{$_} } @{$orgHash->{$pi}};
  $self->{'list_i'} = [@hlist_i];
}

sub getList_i {
  my ($self, $rest) = @_;
  return $self->{'list_i'};
}

sub setList_j {
  my ($self, $pi) = @_;
  my $balancedhash = $self->{'balancedhash'};
  my $orgHash = $self->{'orgHash'};
  my @hlist_j = map { $balancedhash->{$_} } @{$orgHash->{$pi}};
  $self->{'list_j'} = [@hlist_j];
}

sub getList_j {
  my ($self, $rest) = @_;
  return $self->{'list_j'};
}

# Code:
# 0 - No relation
# 1 - $i low -> $j high
# 2 - $i low -> $j low
# 3 - $i high -> $j high
# 4 - $i high -> $j low
# 5 - Equivalent
# 6 - Opposite
sub getCode {
    my ($self, $i, $j) = @_;
    my $code = $self->{'network'}->readCode_1_1($i, $j);
    return $code;
}

sub getNumberCode {
  my $str = shift;
  return 1 if ($str eq "lohi");
  return 2 if ($str eq "lolo");
  return 3 if ($str eq "hihi");
  return 4 if ($str eq "hilo");
  return 5 if ($str eq "equ");
  return 6 if ($str eq "opo");
  return 0;
}

sub getStrCode {
  my $num = shift;
  return "lohi" if ($num == 1);
  return "lolo" if ($num == 2);
  return "hihi" if ($num == 3);
  return "hilo" if ($num == 4);
  return "eqv" if ($num == 5);
  return "opo" if ($num == 6);
  return "";
}

sub getSetConserved {
  my ($self, $type) = @_;
  my $n = $self->{'network'};
  my $conserved = 0;
  my $hi_saved = 0;
  my $hj_saved = 0;
  foreach my $hi (@{$self->{'list_i'}}) {
    foreach my $hj (@{$self->{'list_j'}}) {
      my $c = $self->getCode($hi, $hj);
      if ($c == $type) {
        $conserved = 1;
        $hi_saved = $hi;
        $hj_saved = $hj;
      }
      last if ($conserved == 1);
    }
    last if ($conserved == 1);
  }
  if ($conserved == 1) {
    $self->{'i_saved'} = $hi_saved;
    $self->{'j_saved'} = $hj_saved;
  }
  else {
    $self->{'i_saved'} = undef;
    $self->{'j_saved'} = undef;
  }
  return $conserved;
}

sub getConserved {
  my ($self, $rest) = @_;
  my $pi = $self->pIndex($self->{'i_saved'});
  my $pj = $self->pIndex($self->{'j_saved'});
  my $nameHash = $self->{'nameHash'};
  return join("\t", $pi, $pj, $nameHash->{$pi}, $nameHash->{$pj});
}

sub getName {
  my ($self, $pi) = @_;
  my $nameHash = $self->{'nameHash'};
  return $nameHash->{$pi};
}

sub getRelations {
  my ($self, $pi) = @_;
  if (defined $self->{'relationCache'}->{$pi}) {
    return $self->{'relationCache'}->{$pi};
  }
  my $n = $self->{'network'};
  my $num = $n->getNum();
  my $numrel = 0;
  my $relations = {};
  my $i = $self->getIndex($pi);
  for (my $j = 0; $j < $num/2; $j++) {
    my $code = $self->getCode($i, $j);
    if ($code > 0) {
      if (!defined $relations->{$code}) {
        $relations->{$code} = [];
      }
      push @{$relations->{$code}}, $j;
      $numrel++;
    }
  }
  $relations->{'numrel'} = $numrel;
  $self->{'relationCache'}->{$pi} = $relations;
  my $size = scalar(keys %{$self->{'relationCache'}});
  if ($size > $self->{'cachesize'}) {
    my @list = keys %{$self->{'relationCache'}};
    my $num = int(rand($#list));
    delete $self->{'relationCache'}->{$list[$num]};
    print STDERR "deleting $list[$num]\n";
  }
  return $relations;
}

sub intersection {
  my ($a, $b) = @_;
  my $hash = {};
  my $res = [];
  foreach (@$a) {
    if (!defined $hash->{$_}) {
        $hash->{$_} = 1;
    }
  }
  foreach (@$b) {
    if (defined $hash->{$_}) {
        push @$res, $_;
    }
  }
  return $res;
}

sub union {
  my ($a, $b) = @_;
  my $hash = {};
  my $res = [];
  foreach (@$a) {
    if (!defined $hash->{$_}) {
        $hash->{$_} = 1;
        push @$res, $_;
    }
  }
  foreach (@$b) {
    if (!defined $hash->{$_}) {
        $hash->{$_} = 1;
        push @$res, $_;
    }
  }
  return $res;
}

sub intersectHash {
  my ($a, $b) = @_;
  my $hash = {};
  foreach (keys %{$a}) {
    if (defined $b->{$_}) {
      $hash->{$_} = $a->{$_};
    }
  }
  return $hash;
}

sub unionHash {
  my ($a, $b) = @_;
  my $hash = {};
  foreach (keys %{$a}) {
    $hash->{$_} = $a->{$_};
  }
  foreach (keys %{$b}) {
    $hash->{$_} = $b->{$_};
  }
  return $hash;
}

sub diffHash {
  my ($a, $b) = @_;
  my $hash = {};
  foreach (keys %{$a}) {
    if (!defined $b->{$_}) {
      $hash->{$_} = $a->{$_};
    }
  }
  return $hash;
}

sub _uName {
  my $str = shift;
  if ($str eq "---") {
    return "ZUnknown";
  }
  return $str;
}
sub sortNames {
  my ($self, $arr) = @_;
  return [sort { &_uName($self->getName($a)) cmp &_uName($self->getName($b)) } @$arr];
}

sub MiDReG {
  my ($self, $pi, $pj) = @_;
  my $ri = $self->getRelations($pi);
  my $rj = $self->getRelations($pj);
  #print "Number ".$ri->{'numrel'}." ".$rj->{'numrel'}."\n";
  my $a = &union($ri->{4}, $ri->{6});
  my $b = $rj->{3};
  #print scalar(@$a)." ".scalar(@$b)."\n";
  my $res = &intersection($a, $b);
  my $eqi = [map { $self->pIndex($_) } @{$ri->{5}}];
  my $eqj = [map { $self->pIndex($_) } @{$rj->{5}}];
  my $presj = [map { $self->pIndex($_) } @$res];
  my $a = $ri->{3};
  my $b = &union($rj->{4}, $rj->{6});
  #print scalar(@$a)." ".scalar(@$b)."\n";
  my $res = &intersection($a, $b);
  my $presi = [map { $self->pIndex($_) } @$res];
  $eqi = $self->sortNames($eqi);
  $presi = $self->sortNames($presi);
  $presj = $self->sortNames($presj);
  $eqj = $self->sortNames($eqj);
  return [$eqi, $presi, $presj, $eqj];
}

sub getOrthologPair {
  my ($hn, $array1, $array2) = @_;
  my $res = [];
  my $hash = {};
  foreach my $i (@$array2) {
    $hash->{$i} = 1;
  } 
  foreach my $i (@$array1) {
    my $o = $hn->getOrthologs($i);
    foreach my $j (@$o) {
      if (defined $hash->{$j}) {
        push @{$res}, [$i, $j];
        last;
      }
    }
  }
  return [sort { &_uName($hn->getName($a->[0])) cmp &_uName($hn->getName($b->[0])) } @$res];
}

sub cMiDReG {
  my ($hn, $pi, $pj, $mn, $pi2, $pj2) = @_;
  my $m1 = $hn->MiDReG($pi, $pj);
  my $m2 = $mn->MiDReG($pi2, $pj2);
  #$hn->populateOrthologs($mn->{'type'});
  my $res0 = &getOrthologPair($hn, $m1->[0], $m2->[0]);
  my $res1 = &getOrthologPair($hn, $m1->[1], $m2->[1]);
  my $res2 = &getOrthologPair($hn, $m1->[2], $m2->[2]);
  my $res3 = &getOrthologPair($hn, $m1->[3], $m2->[3]);
  return [$res0, $res1, $res2, $res3];
}

sub _recMiDReG {
  my ($hn, $pi, $pj, $mn, $pi2, $pj2, $hash) = @_;
  my $key = "$pi,$pj,$pi2,$pj2";
  if (defined $hash->{$key}) {
    return $hash;
  }
  my $res = &cMiDReG($hn, $pi, $pj, $mn, $pi2, $pj2);
  $hash->{$key} = $res;
  print "$pi\-$pi2 hilo $pj\-$pj2\n";
  foreach (@{$res->[0]}) {
    print "$pi\-$pi2 equ ". $_->[0]."-".$_->[1]."\n";
  }
  foreach (@{$res->[1]}) {
    print "$pi\-$pi2 hihi ". $_->[0]."-".$_->[1]."\n";
    &_recMiDReG($hn, $_->[0], $pj, $mn, $_->[1], $pj2, $hash);
  } 
  foreach (@{$res->[2]}) {
    print "$pj\-$pj2 hihi ". $_->[0]."-".$_->[1]."\n";
    &_recMiDReG($hn, $pi, $_->[0], $mn, $pi2, $_->[1], $hash);
  } 
  foreach (@{$res->[3]}) {
    print "$pj\-$pj2 equ ". $_->[0]."-".$_->[1]."\n";
  }
  return $hash;
}

sub recMiDReG {
  my ($hn, $pi, $pj, $mn, $pi2, $pj2) = @_;
  my $hash = &_recMiDReG($hn, $pi, $pj, $mn, $pi2, $pj2, {});
}

sub getEquivalent {
  my ($self, $pi, $pj) = @_;
  my $ri = $self->getRelations($pi);
  my $queue = $ri->{5};
  my $res = {};
  while ( scalar(@$queue) != 0) {
    my $item = shift(@$queue);
    next if (defined $res->{$item});
    $res->{$item} = 1;
    my $r = $self->getRelations($self->pIndex($item));
    $q = &union($r->{5}, $queue);
    foreach (@$q) {
        if (!defined $res->{$_}) {
            push @$queue, $_;
        }
    }
  }
  my $pres = [map { $self->pIndex($_) } keys %$res];
  return $pres;
}

sub getIDpairs {
  my $file = shift;
  my $res = [];
  my $hash1 = {};
  my $hash2 = {};
  if (!-e $file) {
    return [$res, $hash1, $hash2];
  }
  my $fh;
  open($fh, "<$file") || die "Can't open $file\n";
  while (<$fh>) {
    s/[\r\n]//g;
    my ($pi, $pj, @rest) = split("\t");
    push @$res, [$pi, $pj];
    if (!defined $hash1->{$pi}) {
      $hash1->{$pi} = {};
    }
    if (!defined $hash2->{$pj}) {
      $hash2->{$pj} = {};
    }
    $hash1->{$pi}->{$pj} = 1;
    $hash2->{$pj}->{$pi} = 1;
  }
  close($fh);
  return [$res, $hash1, $hash2];
}

sub printFileSif {
  my ($hn, $file) = @_;
  my $hash = Network::getIDHash($file);
  foreach my $pi (keys %{$hash}) {
    next if !$hn->isBalanced($pi);
    my $i = $hn->getIndex($pi);
    foreach my $pj (keys %{$hash}) {
      next if ($pi eq $pj);
      next if !$hn->isBalanced($pj);
      my $j = $hn->getIndex($pj);
      next if ($i == $j);
# 1 - $i low -> $j high
# 2 - $i low -> $j low
# 3 - $i high -> $j high
# 4 - $i high -> $j low
# 5 - Equivalent
# 6 - Opposite
      my $code = $hn->getCode($i, $j);
      if ($code == 3) {
        print "$pi hihi $pj\n";
      }
      if ($code == 4) {
        print "$pi hilo $pj\n";
      }
      if ($code == 6) {
        print "$pi hilo $pj\n";
      }
      if ($code == 5) {
        print "$pi equ $pj\n";
      }
    }
  }
}

sub printMiDReGFileSif {
  my ($hn, $file) = @_;
  my $res = &getIDpairs($file);
  my ($pairs, $hash1, $hash2) = @$res;
  my $num = scalar(@$pairs);
  my $index = 0;
  foreach (@$pairs) {
    my $pi = $_->[0];
    my $pj = $_->[1];
    next if !$hn->isBalanced($pi);
    next if !$hn->isBalanced($pj);
    if ( ($index % 10) == 0) {
      print STDERR "Finished $index/$num ...\n";
    }
    $index++;
    my $res = $hn->MiDReG($pi, $pj);
    foreach (@{$res->[0]}) {
      next if (!defined $hash1->{$_});
      print "$pi equ $_\n";
    }
    foreach (@{$res->[1]}) {
      next if (!defined $hash1->{$_});
      print "$pi hihi $_\n";
      print "$pj hilo $_\n";
    }
    foreach (@{$res->[2]}) {
      next if (!defined $hash2->{$_});
      print "$pj hihi $_\n";
      print "$pi hilo $_\n";
    }
    foreach (@{$res->[3]}) {
      next if (!defined $hash2->{$_});
      print "$pj equ $_\n";
    }
  }
}

sub printMiDReGSif {
  my ($hn, $pi, $pj) = @_;
  my $res = $hn->MiDReG($pi, $pj);
  my ($eqi, $hihi_i, $hihi_j, $eqj) = @$res;
  foreach(@$hihi_j) {
    print "$pj hihi $_\n";
  }
  foreach my $pi (@$hihi_j) {
    my $i = $hn->getIndex($pi);
    foreach my $pj (@$hihi_j) {
      my $j = $hn->getIndex($pj);
      next if ($i == $j);
      my $code = $hn->getCode($i, $j);
      if ($code == 3) {
        print "$pi hihi $pj\n";
      }
      if ($code == 5) {
        print "$pi equ $pj\n";
      }
    }
  }
}

use Graph;

sub getMiDReGGraph {
  my ($hn, $pi, $pj) = @_;
  my $res = $hn->MiDReG($pi, $pj);
  my ($eqi, $hihi_i, $hihi_j, $eqj) = @$res;
  my $g = Graph->new();
  foreach(@$hihi_j) {
    $g->addEdge($pj, $_, "hihi");
  }
  foreach my $pi (@$hihi_j) {
    my $i = $hn->getIndex($pi);
    foreach my $pj (@$hihi_j) {
      my $j = $hn->getIndex($pj);
      next if ($i == $j);
      my $code = $hn->getCode($i, $j);
      if ($code == 3) {
        $g->addEdge($pi, $pj, "hihi");
      }
      if ($code == 5) {
        $g->addEdge($pi, $pj, "equ");
      }
    }
  }
  return $g;
}

sub createGraphSIFaddNames {
  my ($hn, $siffile) = @_;
  my $g = Graph->new(file => $siffile); 
  $g->init();
  my ($nodes, $edges) = $g->getNodesEdges("All");
  foreach my $i (keys %{$nodes}) {
    my $pi = $g->getNode($i);
    my ($hi, $mi) = split("-", $pi);
    $g->updateNode($pi, $hn->getName($hi));
  }
  return $g;
}

sub createGraphSIFPairs {
  my ($hn, $file, $siffile) = @_;
  my $res = &getIDpairs($file);
  my ($pairs, $hash1, $hash2) = @$res;
  my $g = &createGraphSIFaddNames($hn, $siffile);
  foreach (@$pairs) {
    my $pi = $_->[0];
    my $pj = $_->[1];
    my ($hi, $mi) = split("-", $pi);
    my ($hj, $mj) = split("-", $pj);
    $g->updateNode($pi, $hn->getName($hi));
    $g->updateNode($pj, $hn->getName($hj));
    $g->addEdge($pi, $pj, "hilo");
  }
  return ($g, $pairs, $hash1, $hash2);
}

sub placeListPairs {
  my ($r, $hash1, $hash2) = @_;
  print STDERR "BFS hihi..\n";
  my $depth = $r->BFS("hihi", 0, -1);
  print STDERR "Placing ..\n";
  my @list1 = grep { defined $hash1->{$r->getNode($_)} } keys(%{$depth});
  my @list2 = grep { defined $hash2->{$r->getNode($_)} } keys(%{$depth});
  my $w = 100;
  my $place1 = $r->placeSideTree(3*$w, 0, $w, $w, \@list1, "hihi", $depth);
  my $place2 = $r->placeSideTree(0, 0, -$w, $w, \@list2, "hihi", $depth);
  my $placed = {};
  my $upList = {};
  my $downList = {};
  foreach my $n (keys %{$place1}) {
    $placed->{$n} = $place1->{$n};
    if ($place1->{$n}->[0] == 3*$w) {
      $upList->{$n} = 1;
    }
  }
  foreach my $n (keys %{$place2}) {
    $placed->{$n} = $place2->{$n};
    if ($place2->{$n}->[0] == 0) {
      $downList->{$n} = 1;
    }
  }
  foreach my $n (keys %{$placed}) {
    print STDERR $r->getNode($n), "\t", $n, "\t", $depth->{$n}, 
          " -> [", $placed->{$n}->[0] , ", " , $placed->{$n}->[1] , "]\n";
  }
  my ($nodes, $edges) = $r->getNodesEdges("hilo");
  my $delPairs = [];
  foreach my $n (keys(%{$edges})) {
    foreach my $j (keys(%{$edges->{$n}})) {
      if (defined $edges->{$j} && defined $edges->{$j}->{$n}) {
        if ($n > $j) {
          push @{$delPairs}, [$n, $j];
        }
      }
      if (!defined $upList->{$n} && ! defined $upList->{$j}
          && !defined $downList->{$n} && ! defined $downList->{$j}) {
        push @{$delPairs}, [$n, $j];
      }
      if (defined $upList->{$n} && ! defined $downList->{$j}) {
        push @{$delPairs}, [$n, $j];
      }
      if (defined $downList->{$n} && ! defined $upList->{$j}) {
        push @{$delPairs}, [$n, $j];
      }
      if (!defined $upList->{$n} && defined $downList->{$j}) {
        push @{$delPairs}, [$n, $j];
      }
      if (!defined $downList->{$n} && defined $upList->{$j}) {
        push @{$delPairs}, [$n, $j];
      }
    }
  }
  my $crossEdges = {};
  foreach (@$delPairs) {
    #$r->deleteEdge($r->getNode($_->[0]), $r->getNode($_->[1]), "hilo");
    $crossEdges->{$_->[0]}->{$_->[1]} = 1;
  }
  $placed->{'upList'} = $place1;
  $placed->{'downList'} = $place2;
  $placed->{'upBorder'} = $upList;
  $placed->{'downBorder'} = $downList;
  $placed->{'crossEdges'} = $crossEdges;
  return $placed;
}

sub expandPairs {
  my ($g, $hash1, $hash2) = @_;
  my $res1 = $g->getConnectedNodes($hash1, "hihi");
  my $res2 = $g->getConnectedNodes($hash2, "hihi");
  my $res = &unionHash($res1, $res2);
  my $res3 = &unionHash($res1, $g->getOtherNodes($res));
  $hash1 = &diffHash($res3, $res2);
  $hash2 = $res2;
  return ($hash1, $hash2);
}

sub printFileNetworkGML {
  my ($hn, $file, $siffile) = @_;
  my ($g, $pairs, $hash1, $hash2) = &createGraphSIFPairs($hn,$file,$siffile);
  print STDERR "Merging equ..\n";
  my $e = $g->mergeEdge("equ");
  print STDERR "SCC hihi..\n";
  #$g->createGraphEdges("equ")->writeGML("t.sif");
  my $r = $e->compactSCC("hihi");
  $r->deleteNodesWithEdges("hihi", $hash1, $hash2);
  print STDERR "Reduce hihi..\n";
  $r->transitiveReduce("hihi");
  #$r->write("t.sif");
  ($hash1, $hash2) = &expandPairs($r, $hash1, $hash2);
  my $placed = &placeListPairs($r, $hash1, $hash2);
  $e->writeGML("t.gml", $placed);
  #$r->writeGMLHandle(\*STDOUT, $r->{'nodes'}, $r->{'edges'}, $placed);
  $r->writeTIKZHandle(\*STDOUT, $r->{'nodes'}, $r->{'edges'}, $placed);
}

sub printFileGML {
  my ($hn, $siffile) = @_;
  my $g = &createGraphSIFaddNames($hn, $siffile);
  print STDERR "Merging equ..\n";
  my $e = $g->mergeEdge("equ");
  print STDERR "SCC hihi..\n";
  my $r = $e->compactSCC("hihi");
  print STDERR "Reduce hihi..\n";
  $r->transitiveReduce("hihi");
  $r->writeGMLHandle(\*STDOUT, $r->{'nodes'}, $r->{'edges'});
}

1;
