#!/usr/bin/perl -I /booleanfs/sahoo/scripts

if (scalar(@ARGV) <= 0) {
    print "perl analyze.pl <cmd> <args>\n";
    exit(1);
}

use IPC::Open2;
use U;
use Hegemon;
use Data::Dumper;
use Env;

my $cmd = shift(@ARGV);
if ($cmd eq "request-1") {
    &analyzeRequest1(@ARGV);
}
if ($cmd eq "request-2") {
    &analyzeRequest2(@ARGV);
}
if ($cmd eq "request-3") {
    &analyzeRequest3(@ARGV);
}

sub analyzeRequest1 {

  my $pre = "/booleanfs2/sahoo/Data/RB/Ravi/ravi-2017-rb";
  my $thrf = "$pre\-thr.txt";
  my $h = Hegemon->new(expr => "$pre\-expr.txt",
      idx => "$pre\-idx.txt", thr => $thrf,
      survival => "$pre\-survival.txt");

  my $start = $h->getStart();
  my $end = $h->getEnd();

  my $type = $h->getSurvivalName('c Type');
  my $normal = [ grep { $type->[$_] eq "Normal Retina"} $start .. $end];
  my $inv = [ grep { $type->[$_] eq "Invasive RB"} $start .. $end];
  my $noninv = [ grep { $type->[$_] eq "Non-invasive RB"} $start .. $end];

  my $pG = [ ["Normal", "green", $normal],
    ["RB", "red", [@$inv, @$noninv]]];
  my $filename = "figures/ravi-normal-cancer.txt";
  $h->setupArrayListData($filename, $pG);

  if (0) {
    my $res = $h->getJavaDiff("$pre\-expr.txt", $filename,
      "/booleanfs2/sahoo/Hegemon/");
    my $res_sorted = [ sort { $res->[$a]->[1] <=> $res->[$b]->[1] } 0 .. $#{$res} ];
    #Diff, pvalue, num1, num2, id, name
    my $i = $res_sorted->[0];
    foreach my $j (@$res_sorted) {
      if ($res->[$j]->[1] != 0) {
        $i = $j;
        last;
      }
    }
    my $maxy = -log($res->[$i]->[1])/log(10);
    foreach my $j (@$res_sorted) {
      next if (abs($res->[$j]->[0]) < 2);
      my $v = $maxy;
      if ($res->[$j]->[1] > 0) {
        $v = -log($res->[$j]->[1])/log(10);
        next;
      }
      print join("\t", sprintf("%.2f\t%0.2f", $res->[$j]->[0], $v), map { $res->[$j]->[$_] } 2 .. 5), "\n";
    }
  }

  my $pre = "/booleanfs2/sahoo/Data/RB/project-1/retina-rb";
  my $thrf = "$pre\-thr.txt";
  my $h = Hegemon->new(expr => "$pre\-expr.txt",
      idx => "$pre\-idx.txt", thr => $thrf,
      survival => "$pre\-survival.txt");

  my $start = $h->getStart();
  my $end = $h->getEnd();

  my $type = $h->getSurvivalName('c Type');
  my $age = $h->getSurvivalName('c Condition');
  my $normal = [ grep { $type->[$_] eq "Control Retina" && $age->[$_] ne 5 } $start .. $end];
  my $rb = [ grep { $type->[$_] eq "RB"} $start .. $end];

  my $pG = [ ["Normal", "green", $normal],
    ["RB", "red", $rb]];
  my $filename = "figures/sailaja-normal-cancer.txt";
  $h->setupArrayListData($filename, $pG);

}

sub analyzeRequest2 {

  my $file = "figures/ravi-normal-cancer-diff.txt";
  open(my $fh, "<$file") || die "Can't open $file\n";
  my $hash1 = {};
  while (<$fh>) {
    my @list = split("\t");
    next if ($list[3] eq 0 || $list[4] eq 0);
    next if (abs($list[0]) < 2);
    next if ($list[1] > 0.05);
    #print join("\t", @list), "\n";
    my ($name, $desc) = split(/: /, $list[5]);
    foreach my $n (split(" /// ", $name)) {
      $n = uc($n);
      if (!defined $hash1->{$n}) {
        $hash1->{$n} = [@list];
      }
      elsif ($hash1->{$n}->[1] > $list[1]) {
        $hash1->{$n} = [@list];
      }
    }
  }
  close($fh);

  my $file = "figures/sailaja-normal-cancer-diff.txt";
  open(my $fh, "<$file") || die "Can't open $file\n";
  my $hash2 = {};
  while (<$fh>) {
    my @list = split("\t");
    next if ($list[3] eq 0 || $list[4] eq 0);
    next if (abs($list[0]) < 2);
    next if ($list[1] > 0.05);
    #print join("\t", @list), "\n";
    my ($name, $desc) = split(/: /, $list[5]);
    foreach my $n (split(" /// ", $name)) {
      $n = uc($n);
      if (!defined $hash2->{$n}) {
        $hash2->{$n} = [@list];
      }
      elsif ($hash2->{$n}->[1] > $list[1]) {
        $hash2->{$n} = [@list];
      }
    }
  }
  close($fh);

  foreach my $n (keys %{$hash1}) {
    next if (!defined $hash2->{$n});
    print join("\t", $hash1->{$n}->[0], $hash2->{$n}->[0], $n), "\n";
  }

}

sub analyzeRequest3 {
  my $pre = "/booleanfs2/sahoo/Data/Public/GPL570/human-gpl570";
  my $thrf = "$pre\-thr.txt";
  my $h = Hegemon->new(expr => "$pre\-expr.txt",
      idx => "$pre\-idx.txt", thr => $thrf,
      survival => "$pre\-survival.txt");

  my $start = $h->getStart();
  my $end = $h->getEnd();

  my $f = "$pre\-vinfo.txt";
  open(my $fh, "<$f") || die "Can't open $f\n";
  my $head = <$fh>;
  $head =~ s/[\r\n]//g;
  my @headers = split("\t", $head);
  my $hhash = {};
  for (my $i = 0; $i < scalar(@headers); $i++) {
    $hhash->{$headers[$i]} = $i;
  }
  my $dhash = {};
  my $data = [];
  while (<$fh>) {
    s/[\r\n]//g;
    my @list = split("\t");
    my $dyn = $list[$hhash->{"Perc 0.90"}] - $list[$hhash->{"Perc -0.90"}];
    my $name = $h->getName($list[0]);
    if ($name =~ /CCDC88A/) {
      print join("\t", $list[0], $dyn, $name), "\n";
    }
    if ($name =~ /CDX/) {
      print join("\t", $list[0], $dyn, $name), "\n";
    }
    $dhash->{$list[0]} = $dyn;
    push @$data, $dyn;
  }
  close($fh);
  $data = [sort { $a <=> $b } @$data];
  my $res = &U::fitStep($data, 0, scalar(@$data) - 1);
  my $thr = [$res->[6], $res->[3], $res->[6]-0.5, $res->[6]+0.5];
  print join("\t", @$thr), "\n";
  #my $ddata = [grep { $_ > $thr->[0] } @$data];
  #$ddata = [sort { $a <=> $b } @$ddata];
  #my $res = &U::fitStep($ddata, 0, scalar(@$ddata) - 1);
  #my $thr = [$res->[6], $res->[3], $res->[6]-0.5, $res->[6]+0.5];
  #print join("\t", @$thr), "\n";

  my @ids = keys(%{$h->{"idHash"}});
  my $nhash = {};
  foreach my $id (@ids) {
    next if (defined $dhash->{$id} && $dhash->{$id} < $thr->[2]);
    my $name = $h->getName($id);
    next if ($name eq "---");
    $name = (split(" /// ", $name))[0];
    push @{$nhash->{$name}}, [$id, $dhash->{$id}];
  }
  my $finalIDs = [];
  open(my $ofh, "> figures/human-gpl570-select.txt");
  print $ofh join("\t", "ArrayID", "DynRange", "Name"), "\n";
  foreach my $name (keys %{$nhash}) {
    my $l = [sort { $b->[1] <=> $a->[1] } @{$nhash->{$name}}];
    my ($id, $dyn) = @{$l->[0]};
    push @$finalIDs, $id;
    print $ofh join("\t", $id, $dyn, $name, map { $_->[1] } @$l), "\n";
  }
  close($ofh);

}

