
# coding: utf-8

# In[1]:

import cv2
import re
import numpy as np
import matplotlib
#matplotlib.use('agg')
from matplotlib import pyplot as plt
from mpl_toolkits.axes_grid1 import make_axes_locatable, axes_size
from mpl_toolkits.axes_grid1 import SubplotDivider, Size
from mpl_toolkits.axes_grid1.mpl_axes import Axes
import matplotlib.patches as patches
import matplotlib.colors as colors
from matplotlib.transforms import *
import PIL
import math
#get_ipython().magic(u'matplotlib inline')
import pandas as pd
import seaborn as sns
import json
from sklearn.metrics import *
from scipy.stats import fisher_exact
from pprint import pprint
import os
import pickle
import sys
sys.path.append("/booleanfs2/sahoo/Hegemon/")
import StepMiner as smn
import HegemonUtil as hu

try:
    reload  # Python 2.7
except NameError:
    try:
        from importlib import reload  # Python 3.4+
    except ImportError:
        from imp import reload  # Python 3.0 - 3.3

acolor = ["#00CC00", "#D8A03D","#EC008C",
        'cyan', "#B741DC", "#808285",
        'blue', 'black', 'green', 'red']

def reactome(idlist):
    import requests
    reactomeURI = 'http://www.reactome.org/AnalysisService/identifiers/projection?pageSize=100&page=1';
    response = requests.post(reactomeURI, data = idlist, \
                headers = { "Content-Type": "text/plain",  "dataType" : "json" })
    obj = json.loads(response.text)
    df = pd.DataFrame()
    df['name'] = [p["name"] for  p in obj["pathways"]]
    df['pValue'] = [p["entities"]["pValue"] for  p in obj["pathways"]]
    df['fdr'] = [p["entities"]["fdr"] for  p in obj["pathways"]]
    return df

def printOLS(fm, df1):
    import statsmodels.formula.api as smf
    lm1 = smf.ols(formula=fm, data=df1).fit()
    print (lm1.summary())
    idx = lm1.params.index
    ci = lm1.conf_int()
    ci_1 = [ "(%0.2f-%0.2f)" % (ci[0][i], ci[1][i]) for i in range(len(idx))]
    df = pd.DataFrame({'Coeff' : lm1.params, '95% CI' : ci_1,
        'pvalues' : lm1.pvalues}, columns=['Coeff', '95% CI' , 'pvalues'])
    for i in range(len(idx)):
        print ('%s\t%.2f\t(%0.2f - %0.2f)\t%0.3f' % \
                (idx[i], lm1.params[i], ci[0][i], ci[1][i], lm1.pvalues[i]))
    print (df.to_string(formatters={'Coeff':'{:,.2f}'.format,
        'pvalues': '{:,.3f}'.format}))

def printStats(cfile, thr):
    if not os.path.isfile(cfile):
        print ("Can't open file {0} <br>".format(cfile))
        exit()
    fp = open(cfile, "r")
    numhigh = 0
    numlow = 0
    total = 0
    for line in fp:
        line = line.strip();
        ll = re.split("[\t]", line);
        if float(ll[2]) >= 0 and float(ll[3]) < thr:
            numhigh += 1
        if float(ll[2]) < 0  and float(ll[3]) < thr:
            numlow += 1
        total += 1
    fp.close();
    print (cfile, numhigh, numlow, total)

def getStats(cfile, thr, index):
    if not os.path.isfile(cfile):
        print ("Can't open file {0} <br>".format(cfile))
        exit()
    fp = open(cfile, "r")
    high = set()
    low = set()
    for line in fp:
        line = line.strip();
        ll = re.split("[\t]", line);
        if float(ll[2]) >= 0 and float(ll[3]) < thr:
            high.add(ll[index])
        if float(ll[2]) < 0  and float(ll[3]) < thr:
            low.add(ll[index])
    fp.close();
    return high, low

def getEntries(cfile, index):
    if not os.path.isfile(cfile):
        print ("Can't open file {0} <br>".format(cfile))
        exit()
    fp = open(cfile, "r")
    res = []
    for line in fp:
        line = line.strip();
        ll = re.split("[\t]", line);
        res += [ll[index]]
    fp.close();
    return res

def getPVal(cfile):
    return [float(i) for i in getEntries(cfile, 3)]

def getFdrStats(cfile, thr, index):
    pval = getPVal(cfile)
    ids = getEntries(cfile, index)
    stat = getEntries(cfile, 2)
    from statsmodels.stats import multitest
    mstat = multitest.multipletests(pval, thr, 'fdr_bh')
    high = set()
    low = set()
    for i in range(len(pval)):
        if mstat[0][i] and float(stat[i]) >= 0:
            high.add(ids[i])
        if mstat[0][i] and float(stat[i]) < 0:
            low.add(ids[i])
    return high, low

def readGenes(cfile):
    genes = "PRKAB1 PPARG PPP1CA PPARGC1A SIRT6 OCLN PARD3 IL23A IL6 S1PR5 ACTA2     CCDC88A COL1A1 CXCL10 ELMO1 HIF1A IL10 IL33 ITGA4 ITGB1 ITGB7 JAK1 MMP14 MMP2     MMP9 MRC1 NOD2 PML PRKCQ RIPK2 S1PR1 SNAI2 SPHK1 TGFB1 TIMP2 TLR2 TLR4 VIM CLDN2     IL11 MMP1 MMP3 CEMIP KIAA1199 PRKAA2 IL8 CXCL8 LGR5"
    if not os.path.isfile(cfile):
        print ("Can't open file {0} <br>".format(cfile))
        exit()
    fp = open(cfile, "r")
    nodelist = re.split("[\[\]()\s]", genes)
    for line in fp:
        line = line.strip();
        ll = re.split("[\[\]()\s]", line);
        nodelist += ll
    fp.close();
    return [i for i in hu.uniq(nodelist) if i != '']

def plotSizes(csizes):
    w,h, dpi = (6, 4, 100)
    fig = plt.figure(figsize=(w,h), dpi=dpi)
    ax = fig.add_axes([70.0/w/dpi, 54.0/h/dpi, 1-2*70.0/w/dpi, 1-2*54.0/h/dpi])
    ax.loglog(range(len(csizes)), csizes, "r-", clip_on=False);
    ax.grid(False)
    ax.set_axis_bgcolor("white")
    for child in ax.get_children():
        if isinstance(child, matplotlib.spines.Spine):
            child.set_color('black')
            child.set_linewidth(0.5)
    ax.tick_params(direction='out', length=4, width=1, colors='k', top=False,
            right=False)
    ax.tick_params(which="minor", direction='out', length=2, width=0.5, 
                   colors='k', top=False, right=False)
    ax.set_xlabel("Clusters ranked by size")
    ax.set_ylabel("Cluster sizes")
    fig.savefig("Supplementary/cluster-sizes-1.pdf", dpi=200)

def printReport(actual, predicted, score, target_names):
    print(classification_report(actual, predicted, target_names=target_names))
    fpr, tpr, _ = roc_curve(actual, score)
    roc_auc = auc(fpr, tpr)
    print ('ROC AUC', roc_auc)
    print ('ROC AUC', roc_auc_score(actual, score))
    print ('Accuracy', accuracy_score(actual, predicted))
    wi,hi, dpi = (4, 4, 100)
    fig = plt.figure(figsize=(wi,hi), dpi=dpi)
    ax = fig.add_axes([70.0/wi/dpi, 54.0/hi/dpi, 1-2*70.0/wi/dpi, 1-2*54.0/hi/dpi])
    ax.plot(fpr, tpr, color='darkorange',
            lw=2, label='ROC curve (area = %0.2f)' % roc_auc)
    ax.plot([0, 1], [0, 1], color='navy', lw=2, linestyle='--')
    ax.set_xlim([0.0, 1.0])
    ax.set_ylim([0.0, 1.05])
    ax.set_xlabel('False Positive Rate')
    ax.set_ylabel('True Positive Rate')
    ax.set_title('Receiver operating characteristic')
    ax.legend(loc="lower right")
    return ax

def convertScore(mylist):
    hs = dict()
    for x in mylist:
        if (x not in hs):
            hs[x] = 1
        else:
            hs[x] += 1
    keys = hs.keys()
    values = [0] + list(np.cumsum(hs.values()))
    for i in range(len(keys)):
        hs[keys[i]] = values[i]
    hh = dict()
    res = []
    for x in mylist:
        if (x not in hh):
            hh[x] = 0
        else:
            hh[x] += 1
        res += [hh[x] + hs[x]]
    return res

def mergeRanks(group, start, exp, weight):
    X = np.array([[e[k-start] for e in exp] for k in group])
    arr = np.dot(X, np.array(weight))
    return arr

def mergeRanks2(group, exp, weight):
    X = np.array([[e[k] for e in exp] for k in range(len(group))])
    arr = np.dot(X, np.array(weight))
    return arr

def getOrder(group, start, exp, weight):
    arr = mergeRanks(group, start, exp, weight)
    return [group[i] for i in np.argsort(arr)]

def getRanks(gene_groups, h):
    expr = []
    row_labels = []
    ranks = []
    counts = []
    for s in gene_groups:
        print (len(s), s)
        count = 0
        avgrank = [0 for i in h.aRange()]
        for gn in s:
          for id in h.getIDs(gn):
            e = h.getExprData(id);
            t = h.getThrData(id);
            if e[-1] == "":
                continue
            v = np.array([float(e[i]) for i in h.aRange()])
            te = []
            for i in h.aRange():
                v1 = (float(e[i]) - t[3]) / 3;
                if np.std(v) > 0:
                    v1 = v1 / np.std(v)
                avgrank[i-h.start] += v1
                te.append(v1)
            expr.append(te)
            #row_labels.append(h.getSimpleName(id))
            row_labels.append(gn)
            count += 1
            #if count > 100:
            #    break
        ranks.append(avgrank)
        counts += [count]
    print (counts)
    return ranks, row_labels, expr

def getRanks2(gene_groups, h):
    expr = []
    row_labels = []
    row_ids = []
    row_numhi = []
    ranks = []
    g_ind = 0
    counts = []
    for s in gene_groups:
        count = 0
        avgrank = [0 for i in h.aRange()]
        for gn in s:
          for id in h.getIDs(gn):
            e = h.getExprData(id);
            if 'NA' in e:
                continue
            t = h.getThrData(id);
            if t is None or len(e) != (h.getEnd()+1):
                continue
            v = np.array([float(e[i]) if e[i] != "" else 0 for i in h.aRange()])
            te = []
            sd = np.std(v)
            for i in h.aRange():
                if (e[i] != ""):
                    v1 = (float(e[i]) - t[3]) / 3;
                    if sd > 0:
                        v1 = v1 / sd
                else:
                    v1 = -t[3]/3/sd
                avgrank[i-h.start] += v1
                te.append(v1)
            expr.append(te)
            row_labels.append(h.getSimpleName(id))
            row_ids.append(id)
            v1 = [g_ind, sum(v > t[3])]
            if g_ind > 3:
                v1 = [g_ind, sum(v <= t[3])]
            else:
                v1 = [g_ind, sum(v > t[3])]
            row_numhi.append(v1)
            count += 1
            #if count > 200:
            #    break
        ranks.append(avgrank)
        g_ind += 1
        counts += [count]
    print (counts)
    return ranks, row_labels, row_ids, row_numhi, expr

def getRanks3(gene_groups, h, order):
    expr = []
    row_labels = []
    row_ids = []
    row_numhi = []
    ranks = []
    g_ind = 0
    counts = []
    for s in gene_groups:
        count = 0
        avgrank = [0 for i in order]
        for gn in s:
          for id in h.getIDs(gn):
            e = h.getExprData(id);
            if len(e) != (h.getEnd()+1):
                continue
            v = np.array([float(e[i]) if e[i] != "" else 0 for i in order])
            t = hu.getThrData(v)
            te = []
            for i in range(len(order)):
                if e[order[i]] == "":
                    v1 = - t[3] / 3;
                else:
                    v1 = (float(e[order[i]]) - t[3]) / 3;
                if np.std(v) > 0:
                    v1 = v1 / np.std(v)
                avgrank[i] += v1
                te.append(v1)
            expr.append(te)
            row_labels.append(h.getSimpleName(id))
            row_ids.append(id)
            v1 = [g_ind, sum(v > t[3])]
            if g_ind > 3:
                v1 = [g_ind, sum(v <= t[3])]
            else:
                v1 = [g_ind, sum(v > t[3])]
            row_numhi.append(v1)
            count += 1
            #if count > 200:
            #    break
        ranks.append(avgrank)
        g_ind += 1
        counts += [count]
    print (counts)
    return ranks, row_labels, row_ids, row_numhi, expr

def saveList(ofile, l1):
    of = open(ofile, "w")
    for i in l1:
        of.write("\t".join([i]) + "\n")
    of.close()

def readList(cfile):
    if not os.path.isfile(cfile):
        print ("Can't open file {0} <br>".format(cfile))
        exit()
    fp = open(cfile, "r")
    f_order = []
    for line in fp:
        line = line.strip();
        ll = re.split("[\s]", line);
        f_order += [ll[0]]
    fp.close();
    return f_order

def saveCData(ofile, h, i1, f_ranks):
    f_order = dict()
    for i in h.aRange():
        f_order[i] = ""
    for i in range(len(i1)):
        f_order[i1[i]] = str(i)
    of = open(ofile, "w")
    for i in h.aRange():
        id1 = h.headers[i]
        of.write("\t".join([id1, f_order[i], str(f_ranks[i - h.start])]) + "\n")
    of.close()

def saveHeatmapData(ofile, row_labels, row_numhi, row_ids, index, expr):
    ind_r = np.array(sorted(range(len(row_labels)), key=lambda x: (row_numhi[x][0], row_numhi[x][1])))
    of = open(ofile, "w")
    for i in ind_r:
        id1 = row_ids[i]
        of.write("\t".join([id1, row_labels[i], str(row_numhi[i][0]),                             str(row_numhi[i][1])] + [str(expr[i][j]) for j in index]) + "\n")
    of.close()

def readCData(cfile):
    if not os.path.isfile(cfile):
        print ("Can't open file {0} <br>".format(cfile))
        exit()
    fp = open(cfile, "r")
    f_order = []
    f_ranks = []
    for line in fp:
        line = line.strip();
        ll = re.split("[\s]", line);
        f_order += [ll[1]]
        f_ranks += [float(ll[2])]
    fp.close();
    return f_order, f_ranks

def readHeatmapData(cfile):
    if not os.path.isfile(cfile):
        print ("Can't open file {0} <br>".format(cfile))
        exit()
    fp = open(cfile, "r")
    row_labels, row_numhi, row_ids, expr = [], [], [], []
    for line in fp:
        line = line.strip();
        ll = re.split("\t", line);
        row_ids += [ll[0]]
        row_labels += [ll[1]]
        row_numhi += [ [int(ll[2]), int(ll[3])] ]
        expr += [[float(k) for k in ll[4:]]]
    fp.close();
    return row_labels, row_numhi, row_ids, expr

def barTop(tax, atypes, color_sch1, params):
    spaceAnn = 70
    widthAnn = 3
    tAnn = 1
    if 'spaceAnn' in params:
        spaceAnn = params['spaceAnn']
    if 'widthAnn' in params:
        widthAnn = params['widthAnn']
    if 'tAnn' in params:
        tAnn = params['tAnn']
    for i in range(len(atypes)):
        tax.add_patch(patches.Rectangle( (i *spaceAnn, 0), widthAnn, 3,
                                        facecolor=color_sch1[i], edgecolor="none", alpha=1.0))
        tax.text(i * spaceAnn + widthAnn + tAnn, 1, atypes[i], rotation='horizontal',
                 ha='left', va='center', fontsize=12)

def plotHeatmap(ofile, data, col_labels, row_labels, params):
    
    genes = []
    atypes = []
    cval = []
    dpi, tl, tw, ts, tsi = (100, 3, 0.25, 0.5, 0)
    if 'genes' in params:
        genes = params['genes']
    if 'atypes' in params:
        atypes = params['atypes']
    if 'cval' in params:
        cval = params['cval']
    if 'dpi' in params:
        dpi = params['dpi']
    if 'tl' in params:
        tl = params['tl']
    if 'tw' in params:
        tw = params['tw']
    if 'ts' in params:
        ts = params['ts']
    if 'tsi' in params:
        tsi = params['tsi']
    
    w,h = (12, 12)
    dx, dy = (10, 10)
    if 'dx' in params:
        dx = params['dx']
    if 'dy' in params:
        dy = params['dy']
    if 'w' in params:
        w = params['w']
    if 'h' in params:
        h = params['h']
        
    nAt, nGt = (len(col_labels), len(row_labels))
    if 'ax' not in params:
        fig = plt.figure(figsize=(w,h), dpi=dpi)
        ax = fig.add_axes([70.0/w/dpi, 54.0/h/dpi, 1-2*70.0/w/dpi, 1-2*54.0/h/dpi])
    else:
        ax = params['ax']
    extent = [0, nAt*dx, 0, nGt*dy]

    cvals  = [-1, -0.7, -0.4, 0, 0, 0.2, 1]
    clrs = ["#210B61","#0B614B","#04B45F", "#D8F781", "#F2F5A9", "red", "#DF0101"]
    norm=plt.Normalize(min(cvals),max(cvals))
    tuples = list(zip(map(norm,cvals), clrs))
    cmap = colors.LinearSegmentedColormap.from_list("BGYR1", tuples)
    plt.register_cmap(cmap=cmap)

    cvals  = [-1, -0.7, -0.4, 0, 0, 0.8, 1]
    clrs = ["#210B61","#0B614B","#04B45F", "#D8F781", "#F2F5A9", "red", "#DF0101"]
    norm=plt.Normalize(min(cvals),max(cvals))
    tuples = list(zip(map(norm,cvals), clrs))
    cmap = colors.LinearSegmentedColormap.from_list("BGYR2", tuples)
    plt.register_cmap(cmap=cmap)

    im = ax.imshow(data, cmap="bwr", interpolation='nearest', vmin=-2.0, vmax=2.0, extent = extent)

    ax.tick_params(top=True, bottom=False, labeltop=True, labelbottom=False)
    ax.set_xticklabels([])
    
    yticks = []
    ylabels = []
    for g in genes:
        if g in row_labels:
            i = row_labels.index(g)
            yticks += [-dy/2 + (len(row_labels) - i) * dy]
            ylabels += [ row_labels[i] ]
    si = np.argsort(np.array(yticks))
    yiticks = np.array(yticks)[si]
    yoticks = np.array(yticks)[si]
    ylabels = np.array(ylabels)[si]
    sy = 5
    if 'sy' in params:
        sy = params['sy']
    for i in range(1, len(yoticks)):
        diff = yoticks[i] - yoticks[i - 1]
        if diff < sy*dy:
            yoticks[i] = yoticks[i - 1] + sy*dy
    for i in range(len(yoticks)):
        yoticks[i] = yoticks[i] + tsi
    ax.set_yticks(yiticks)
    ax.set_yticklabels([])
    plt.setp(ax.get_xticklabels(), rotation=-30, ha="right", rotation_mode="anchor")
    for edge, spine in ax.spines.items():
        spine.set_visible(False)
    ax.grid(False)
    ax.tick_params(top=False, left=True, bottom=False, right=False,
            length=tl, width=tw)
    plt.xlim(xmin=0)
    trans = blended_transform_factory(ax.transData, ax.transData)
    fx, fy =  ax.transData.transform((1, 1)) - ax.transData.transform((0, 0))
    fx = dpi/fx/72
    fy = dpi/fy/72
    print (fx, fy)
    fx = max(fx, fy)
    fy = max(fx, fy)
    oo = 2
    for i in range(len(yoticks)):
        ax.annotate(str(ylabels[i]), xy=(0.0, yoticks[i]),
                xycoords=trans,
                xytext=(-(2*tl+ts), 0), textcoords='offset points', color="black",
                fontsize=8, ha="right")
        ax.plot((-(2*tl+ts)*fx, -(tl+ts+oo)*fx, -(tl+oo)*fx, -tl*fx),
                (yoticks[i]+4*fy, yoticks[i]+4*fy, yiticks[i], yiticks[i]),
                transform=trans,
                linestyle='solid', linewidth=tw, color='black', clip_on=False)
        oo += 0.5
        if (oo > 2):
            oo = 0

    # Create colorbar
    aspect = 20
    pad_fraction = 0.5
    divider = make_axes_locatable(ax)
    width = axes_size.AxesY(ax, aspect=1./aspect)
    pad = axes_size.Fraction(pad_fraction, width)
    cax = divider.append_axes("right", size=width, pad=pad)
    cbar = plt.colorbar(im, cax=cax)
    cbar.ax.set_ylabel("Expression", rotation=-90, va="bottom")

    color_sch1 = ["#3B449C", "#B2509E","#EA4824"]
    color_sch1 = ["#00CC00", "#EFF51A","#EC008C", "#F7941D", "#808285",
            'cyan', 'blue', 'black', 'green', 'red']
    if 'acolor' in params:
        color_sch1 = params['acolor']

    if len(cval) > 0:
        width = axes_size.AxesX(ax, aspect=1./aspect)
        pad = axes_size.Fraction(pad_fraction, width)
        tax = divider.append_axes("top", size=width, pad=pad)
        extent = [0, nAt, 0, 5]
        tax.axis(extent)
        cmap = colors.ListedColormap(color_sch1)
        boundaries = range(len(color_sch1) + 1)
        norm = colors.BoundaryNorm(boundaries, cmap.N, clip=True)
        tax.imshow(cval, interpolation='nearest', cmap=cmap, norm=norm, extent=extent, aspect="auto")
        tax.set_xticklabels([])
        tax.set_yticklabels([])
        tax.tick_params(top=False, left=False, bottom=False, right=False)
        if 'tline' in params and params['tline'] == 1:
            tax.set_xticks(np.arange(0, nAt, 1))
            tax.grid(which='major', alpha=1, linestyle='-', linewidth='1',
                    color='black')
        else:
            tax.grid(False)

    pad = axes_size.Fraction(0.2, width)
    lax = divider.append_axes("top", size=width, pad=pad, frame_on=False)
    lax.axison = False
    lax.axis(extent)
    lax.set_xticklabels([])
    lax.set_yticklabels([])
    lax.grid(False)
    lax.tick_params(top=False, left=False, bottom=False, right=False)
    barTop(lax, atypes, color_sch1, params)

    ax.get_figure().savefig(ofile, dpi=dpi)
    return ax

def plotTitleBar(cval, atypes, params):
    dpi = 100
    if 'dpi' in params:
        dpi = params['dpi']
    w,h = (5, 0.8)
    if 'w' in params:
        w = params['w']
    if 'h' in params:
        h = params['h']
    color_sch1 = ["#3B449C", "#B2509E","#EA4824"]
    color_sch1 = ["#00CC00", "#EFF51A","#EC008C", "#F7941D", "#808285",
            'cyan', 'blue', 'black', 'green', 'red']
    if 'acolor' in params:
        color_sch1 = params['acolor']
    if 'cval' in params:
        cval = params['cval']

    ax = None
    if 'ax' in params:
        ax = params['ax']
    if ax is None:
        fig = plt.figure(figsize=(w,h), dpi=dpi)
        ax = fig.add_subplot(1, 1, 1)
    nAt = len(cval[0])
    extent = [0, nAt, 0, 5]
    ax.axis(extent)
    cmap = colors.ListedColormap(color_sch1)
    boundaries = range(len(color_sch1) + 1)
    norm = colors.BoundaryNorm(boundaries, cmap.N, clip=True)
    ax.imshow(cval, interpolation='nearest', cmap=cmap, \
                      norm=norm, extent=extent, aspect="auto")
    ax.set_xticklabels([])
    ax.set_yticklabels([])
    ax.tick_params(top=False, left=False, bottom=False, right=False)
    ax.set_xticks(np.arange(0, nAt, 1))
    ax.grid(which='major', alpha=0.2, linestyle='-', linewidth=0.5,
            color='black')
    for edge, spine in ax.spines.items():
                spine.set_visible(False)
    divider = make_axes_locatable(ax)
    width = axes_size.AxesX(ax, aspect=1./20)
    spaceAnn = 70
    widthAnn = 3
    tAnn = 1
    if 'spaceAnn' in params:
        spaceAnn = params['spaceAnn']
    if 'widthAnn' in params:
        widthAnn = params['widthAnn']
    if 'tAnn' in params:
        tAnn = params['tAnn']
    pad = axes_size.Fraction(0.1, width)
    lax = divider.append_axes("top", size="100%", pad="20%", frame_on=False)
    lax.axison = False
    lax.axis(extent)
    lax.set_xticklabels([])
    lax.set_yticklabels([])
    lax.grid(False)
    lax.tick_params(top=False, left=False, bottom=False, right=False)
    if 'atypes' in params:
        atypes = params['atypes']
    barTop(lax, atypes, color_sch1, params)
    return ax

def getGroupsMm(gene_groups):
    cfile = "/booleanfs2/sahoo/Data/SeqData/genome/Homo_sapiens.GRCh38.95.chr_patch_hapl_scaff.len.txt"
    fp = open(cfile, "r")
    hsdict = {}
    for line in fp:
        line = line.strip();
        ll = re.split("\t", line);
        hsdict[ll[0]] = ll[1]
    fp.close();

    cfile = "/booleanfs2/sahoo/Data/Sarah/CellCycle/database/mart-export-hs-mm.txt"
    fp = open(cfile, "r")
    mmdict = {}
    for line in fp:
        line = line.strip();
        ll = re.split("\t", line);
        if len(ll) > 3 and ll[0] in hsdict:
            g = hsdict[ll[0]]
            if g not in mmdict:
                mmdict[g] = []
            mmdict[g] += [ll[3]]
    fp.close();

    gene_groups_mm = []
    for s in gene_groups:
        s1 = set()
        for g in s:
            if g in mmdict:
                for k in mmdict[g]:
                    s1.add(k)
        gene_groups_mm.append(s1)
    return gene_groups_mm

def getGroupsHs(gene_groups):
    cfile = "/booleanfs2/sahoo/Data/SeqData/genome/Homo_sapiens.GRCh38.95.chr_patch_hapl_scaff.len.txt"
    fp = open(cfile, "r")
    hsdict = {}
    for line in fp:
        line = line.strip();
        ll = re.split("\t", line);
        hsdict[ll[0]] = ll[1]
    fp.close();

    cfile = "/booleanfs2/sahoo/Data/Sarah/CellCycle/database/mart-export-hs-mm.txt"
    fp = open(cfile, "r")
    mmdict = {}
    for line in fp:
        line = line.strip();
        ll = re.split("\t", line);
        if len(ll) > 3 and ll[0] in hsdict:
            g = hsdict[ll[0]]
            if ll[3] not in mmdict:
                mmdict[ll[3]] = []
            mmdict[ll[3]] += [g]
    fp.close();

    gene_groups_hs = []
    for s in gene_groups:
        s1 = set()
        for g in s:
            if g in mmdict:
                for k in mmdict[g]:
                    s1.add(k)
        gene_groups_hs.append(s1)
    return gene_groups_hs

def getGeneGroups(order = None, weight = None, debug = 1):
    reload(hu)
    db = hu.Database("/booleanfs2/sahoo/Hegemon/explore.conf")
    h = hu.Hegemon(db.getDataset("GS18"))
    h.init()
    h.initPlatform()
    h.initSurv()
    data_item = []
    dir1 = "/booleanfs2/sahoo/Data/Pradipta/gastric/BN/"
    with open(dir1 + 'figures-3/path-0.json') as data_file:
        data_item += json.load(data_file)
    with open(dir1 + 'figures-3/path-1.json') as data_file:
        data_item += json.load(data_file)
    with open(dir1 + 'figures-3/path-2.json') as data_file:
        data_item += json.load(data_file)
    cfile = dir1 + "figures/gc-net-3-cls.txt"
    if not os.path.isfile(cfile):
        print ("Can't open file {0} <br>".format(cfile))
        exit()
    fp = open(cfile, "r")
    nodelist = {}
    nhash = {}
    for line in fp:
        line = line.strip();
        ll = line.split("\t");
        nodelist[ll[0]] = ll[2:]
        for i in ll[2:]:
            nhash[i] = ll[0];
    fp.close();
    gene_groups = []
    for i in range(len(data_item)):
        gene_groups.append(set())
        gn = data_item[i][2][0][0]
        for g in data_item[i][2]:
            gene_groups[i].add(g[0])
            if g[0] in nodelist:
                for k in nodelist[g[0]]:
                    gene_groups[i].add(k)
        for g in data_item[i][3]:
            gene_groups[i].add(g)
            if g in nodelist:
                for k in nodelist[g]:
                    gene_groups[i].add(k)
        if debug == 1:
            print (i, gn, h.getSimpleName(gn), data_item[i][0], len(gene_groups[i]))
    print([len(s) for s in gene_groups])
    if order is None:
        order = [1, 3, 4, 5];
        order = [35]
        order = [43, 44, 45];
        order = [8, 9, 10]
    gene_groups = [gene_groups[i] for i in order]
    print([len(s) for s in gene_groups])
    #gene_groups = getSimpleName(gene_groups, h)
    #print([len(s) for s in gene_groups])
    if weight is None:
        weight = [-1, 1, 2, 3]
        weight = [-1, -2, -3]
        weight = [-1]
        weight = [-1, -2, -3]
    print (weight)
    genes = readGenes(dir1 + "figures/cluster-names.txt")
    return genes, weight, gene_groups

def getGeneGroups4(order = None, weight = None, debug = 1):
    reload(hu)
    db = hu.Database("/booleanfs2/sahoo/Hegemon/explore.conf")
    h = hu.Hegemon(db.getDataset("GS17.2"))
    h.init()
    h.initPlatform()
    h.initSurv()
    data_item = []
    dir1 = "/booleanfs2/sahoo/Data/Pradipta/gastric/BN/"
    with open(dir1 + 'figures-4/path-0.json') as data_file:
        data_item += json.load(data_file)
    with open(dir1 + 'figures-4/path-1.json') as data_file:
        data_item += json.load(data_file)
    with open(dir1 + 'figures-4/path-2.json') as data_file:
        data_item += json.load(data_file)
    with open(dir1 + 'figures-4/path-3.json') as data_file:
        data_item += json.load(data_file)
    with open(dir1 + 'figures-4/path-4.json') as data_file:
        data_item += json.load(data_file)
    with open(dir1 + 'figures-4/path-5.json') as data_file:
        data_item += json.load(data_file)
    cfile = dir1 + "figures/gc-net-4-cls.txt"
    if not os.path.isfile(cfile):
        print ("Can't open file {0} <br>".format(cfile))
        exit()
    fp = open(cfile, "r")
    nodelist = {}
    nhash = {}
    for line in fp:
        line = line.strip();
        ll = line.split("\t");
        nodelist[ll[0]] = ll[2:]
        for i in ll[2:]:
            nhash[i] = ll[0];
    fp.close();
    gene_groups = []
    for i in range(len(data_item)):
        gene_groups.append(set())
        gn = data_item[i][2][0][0]
        for g in data_item[i][2]:
            gene_groups[i].add(g[0])
            if g[0] in nodelist:
                for k in nodelist[g[0]]:
                    gene_groups[i].add(k)
        for g in data_item[i][3]:
            l = g.split(" ")
            g = l[0]
            gene_groups[i].add(g)
            if g in nodelist:
                for k in nodelist[g]:
                    gene_groups[i].add(k)
        if debug == 1:
            print (i, gn, h.getSimpleName(gn), data_item[i][0], len(gene_groups[i]))
    print([len(s) for s in gene_groups])
    if order is None:
        order = [1, 3, 4, 5];
        order = [35]
        order = [43, 44, 45];
        order = [8, 9, 10]
    gene_groups = [gene_groups[i] for i in order]
    print([len(s) for s in gene_groups])
    #gene_groups = getSimpleName(gene_groups, h)
    #print([len(s) for s in gene_groups])
    if weight is None:
        weight = [-1, 1, 2, 3]
        weight = [-1, -2, -3]
        weight = [-1]
        weight = [-1, -2, -3]
    print (weight)
    genes = readGenes(dir1 + "figures/cluster-names.txt")
    return genes, weight, gene_groups

def getMacGeneGroups(order = None, weight = None, debug = 1):
    reload(hu)
    db = hu.Database("/booleanfs2/sahoo/Hegemon/explore.conf")
    h = hu.Hegemon(db.getDataset("GL4"))
    h.init()
    h.initPlatform()
    h.initSurv()
    data_item = []
    dir1 = "/booleanfs2/sahoo/Data/Macrophage/BN/"
    with open(dir1 + 'figures/path-1.json') as data_file:
        data_item += json.load(data_file)
    with open(dir1 + 'figures/path-0.json') as data_file:
        l1 = json.load(data_file)
        data_item[5] = l1[5]
        data_item[6] = l1[6]
    with open(dir1 + 'figures/path-2.json') as data_file:
        data_item += json.load(data_file)
    with open(dir1 + 'figures/path-3.json') as data_file:
        data_item += json.load(data_file)
    with open(dir1 + 'figures/path-4.json') as data_file:
        data_item += json.load(data_file)
    cfile = dir1 + "figures/mac-net-cls.txt"
    if not os.path.isfile(cfile):
        print ("Can't open file {0} <br>".format(cfile))
        exit()
    fp = open(cfile, "r")
    nodelist = {}
    nhash = {}
    for line in fp:
        line = line.strip();
        ll = line.split("\t");
        nodelist[ll[0]] = ll[2:]
        for i in ll[2:]:
            nhash[i] = ll[0];
    fp.close();
    gene_groups = []
    for i in range(len(data_item)):
        gene_groups.append(set())
        gn = data_item[i][2][0][0]
        for g in data_item[i][2]:
            gene_groups[i].add(g[0])
            if g[0] in nodelist:
                for k in nodelist[g[0]]:
                    gene_groups[i].add(k)
        for g in data_item[i][3]:
            gene_groups[i].add(g)
            if g in nodelist:
                for k in nodelist[g]:
                    gene_groups[i].add(k)
        if debug == 1:
            print (i, gn, h.getSimpleName(gn), data_item[i][0], len(gene_groups[i]))
    print([len(s) for s in gene_groups])
    if order is None:
        order = [1, 3, 4, 5];
        order = [35]
        order = [43, 44, 45];
        order = [8, 9, 10]
    gene_groups = [gene_groups[i] for i in order]
    print([len(s) for s in gene_groups])
    gene_groups = getSimpleName(gene_groups, h)
    print([len(s) for s in gene_groups])
    if weight is None:
        weight = [-1, 1, 2, 3]
        weight = [-1, -2, -3]
        weight = [-1]
        weight = [-1, -2, -3]
    print (weight)
    genes = readGenes(dir1 + "figures/cluster-names.txt")
    return genes, weight, gene_groups

def getSimpleName(gene_groups, h):
    res = []
    for s in gene_groups:
        s1 = set()
        for g in s:
            for id1 in h.getIDs(g):
                name = h.getSimpleName(id1)
                if name != "" and name != "---":
                    s1.add(name)
        res.append(s1)
    return res

def getCoates():
    cfile = "database/c2.all.v6.2.symbols.gmt"
    fp = open(cfile, "r")
    l1 = l2 = None
    for line in fp:
        line = line.strip();
        ll = line.split("\t");
        if ll[0] == "COATES_MACROPHAGE_M1_VS_M2_UP":
           l1 = ll[2:]
        if ll[0] == "COATES_MACROPHAGE_M1_VS_M2_DN":
           l2 = ll[2:]
    fp.close();
    res = [set(l1), set(l2)]
    return res

def getMartinez():
    cfile = "database/c7.all.v6.2.symbols.gmt"
    fp = open(cfile, "r")
    l1 = l2 = None
    for line in fp:
        line = line.strip();
        ll = line.split("\t");
        if ll[0] == "GSE5099_CLASSICAL_M1_VS_ALTERNATIVE_M2_MACROPHAGE_UP":
           l2 = ll[2:]
        if ll[0] == "GSE5099_CLASSICAL_M1_VS_ALTERNATIVE_M2_MACROPHAGE_DN":
           l1 = ll[2:]
    fp.close();
    res = [set(l1), set(l2)]
    return res

def getMartinezM2hsmm():
    cfile = "database/Martinez-Table3.txt"
    res = getEntries(cfile, 0)
    return [res[1:]]

def getMartinezM0hsmm():
    cfile = "database/Martinez-TableS1.txt"
    res = getEntries(cfile, 0)
    return [res[1:]]

def getBell2016():
# Citation: Bell LCK, Pollara G, Pascoe M, Tomlinson GS, Lehloenya RJ, Roe J, et
# al. (2016) In Vivo Molecular Dissection of the Effects of HIV-1 in Active
# Tuberculosis. PLoS Pathog 12(3): e1005469.
# https://doi.org/10.1371/journal.ppat.1005469
    cfile = "database/journal.ppat.1005469.s018.txt"
    fp = open(cfile, "r")
    l1 = l2 = l3 = None
    for line in fp:
        line = line.strip();
        ll = line.split("\t");
        if ll[0] == "LPS":
           l1 = ll[2].replace('"', "").split(", ")
        if ll[0] == "IFNg":
           l2 = ll[2].replace('"', "").split(", ")
        if ll[0] == "IL-4 and IL-13":
           l3 = ll[2].replace('"', "").split(", ")
    fp.close();
    res = [set(l1), set(l2), set(l3)]
    return res

def getBecker():
    #Citation:
    #Becker M, De Bastiani MA, Parisi MM, Guma FT, Markoski MM, Castro
    #MA, Kaplan MH, Barbé-Tuana FM, Klamt F. Integrated Transcriptomics 
    #Establish Macrophage Polarization Signatures and have Potential 
    #Applications for Clinical Health and Disease. Sci Rep. 2015 Aug 25;
    #5:13351. doi: 10.1038/srep13351.
    cfile = "database/srep13351-s1-1.txt"
    l1 = getEntries(cfile, 0)
    cfile = "database/srep13351-s1-2.txt"
    l2 = getEntries(cfile, 0)
    return [l1[1:], l2[2:]]

def processGeneGroups(ana, l1, wt1, debug = 0, fthr = None):
    ana.orderData(l1, wt1); print ("ROC-AUC", ana.getMetrics())
    actual = [1 if ana.aval[i] >= 1 else 0 for i in ana.i1]
    score = [ana.f_ranks[i - ana.h.start] for i in ana.i1]
    thr = hu.getThrData(score)
    nm = (np.max(ana.f_ranks) - np.min(ana.f_ranks))/16
    if fthr is None:
        fthr = thr[0]
    if fthr == "thr0":
        fthr = thr[0] - nm
    if fthr == "thr2":
        fthr = thr[0] + nm
    if fthr == "thr3":
        fthr = thr[0] + 3 * nm
    print (thr)
    print (nm, fthr)
    predicted = [1 if ana.f_ranks[i - ana.h.start] >= fthr else 0 for i in ana.i1]
    c_dict = {}
    for i in ana.order:
        c_dict[i] = ana.f_ranks[i - ana.h.start]
        c_dict[i] = 0
        if ana.f_ranks[i - ana.h.start] >= fthr:
            c_dict[i] = 1
    fpr, tpr, thrs = roc_curve(actual, score)
    roc_auc = auc(fpr, tpr)
    if debug == 1:
        print (actual)
        print (predicted)
        print (score)
        print (thr[0], thr, nm)
        print ("ROC-AUC", roc_auc)
        print ('Accuracy', accuracy_score(actual, predicted))
        print (classification_report(actual, predicted, target_names=ana.atypes))
    return c_dict, fpr, tpr, roc_auc

def processOneDataset(ana, l1, wt1, prms = None, sp = 'Hs'):
    sel = 2
    if sp == 'Hs':
        ana.orderData(l1, wt1)
    else:
        genes = []
        ana.convertMm(l1, genes)
        ana.orderData(ana.gene_groups, wt1)
    #ana.normMacrophageGene("TYROBP", {"thr":1})
    #ana.normMacrophageGene("FCER1G", {"thr":4, "tval":-4, "select":sel})
    fig = plt.figure(figsize=(4,4), dpi=100)
    ax = plt.subplot2grid((4, 1), (0, 0))
    params = {'spaceAnn': len(ana.order)/len(ana.atypes),
            'tAnn': 1, 'widthAnn':1, 'acolor': acolor,
            'w': 5, 'h': 0.8, 'atypes': ana.atypes ,'cval': ana.cval,
            'ax': ax}
    if prms is not None:
        params.update(prms)
    ax = ana.printTitleBar(params)
    roc = ana.getMetrics()
    print ("ROC-AUC", roc)
    ax.text(len(ana.cval[0]), 4, str(roc))
    ax = plt.subplot2grid((4, 1), (1, 0), rowspan=3)
    ax = ana.densityPlot(ax, acolor)
    plt.tight_layout()
    plt.show()
    return fig

#https://www.ncbi.nlm.nih.gov/pmc/articles/PMC6499450
def getZhang28Gene():
    l1 = [getEntries("database/zhang-2019-28-gene.txt", 0)]
    wt1 = [1]
    return wt1, l1

#https://www.ncbi.nlm.nih.gov/pmc/articles/PMC5650304/
def getWang9Gene():
    l1 = [getEntries("database/wang-2017-9-gene.txt", 3)[1:]]
    wt1 = [1]
    return wt1, l1

#https://www.ncbi.nlm.nih.gov/pubmed/28035468
def getHau11Gene():
    l1 = [getEntries("database/hou-2017-11-gene.txt", 0)]
    wt1 = [-1]
    return wt1, l1

#https://www.ncbi.nlm.nih.gov/pubmed/21445269
def getCui19Gene():
    l1 = [getEntries("database/cui-2011-19-gene.txt", 0)]
    wt1 = [-1]
    return wt1, l1

#https://www.ncbi.nlm.nih.gov/pubmed/25928635
def getLi2015Gene():
    l1 = [getEntries("database/journal.pone.0125013.t001.txt", 0),
            getEntries("database/journal.pone.0125013.t001.txt", 1)]
    wt1 = [-1, 1]
    return wt1, l1

#https://www.ncbi.nlm.nih.gov/pubmed/25500430
def getZang2014():
    l1 = [getEntries("database/zang-2014-down.txt", 3)[1:10],
            getEntries("database/zang-2014-up.txt", 3)[1:10]]
    wt1 = [-1, 1]
    return wt1, l1

#https://www.ncbi.nlm.nih.gov/pubmed/29745833
def getDai2018Gene():
    thr = 1.0
    ifile = "figures/tcga-2019-diff.txt"
    high, low = getFdrStats(ifile, thr, 1)
    l2 = ["GIF", "XBP1", "CEBPA", "GATA6", "DPP4", "PTK7", "GATA3", "FOSB",
            "EPAS1", "CCND2", "MALL", "IRF1", "SOX9", "VILL", "GALNT3",
            "LGALS4"]
    l1 = [high.intersection(l2), low.intersection(l2)]
    wt1 = [-1, 1]
    return wt1, l1

#https://www.ncbi.nlm.nih.gov/pubmed/18827816
def getTakeno2010Gene():
    l1 = [["MMP7", "SPARC", "SOD2", "INHBA", "IGFBP7", "NEK6", "LUM"]]
    wt1 = [1]
    return wt1, l1

#https://www.ncbi.nlm.nih.gov/pubmed/31127138
def getLi2019Deg():
    l1 = [getEntries("database/li-2019-down.txt", 1)[1:],
            getEntries("database/li-2019-up.txt", 1)[1:]]
    wt1 = [-1, 1]
    return wt1, l1

#https://www.ncbi.nlm.nih.gov/pubmed/31127138
def getLi2019Hub():
    l1 = [ ["SST", "ALDH1A1"],
            ["IL8", "COL1A1", "MMP9", "COL1A2", "TIMP1",
            "FN1", "SPARC", "SERPINE1" ] ]
    wt1 = [-1, 1]
    return wt1, l1

def getClsGroups4(v1, wt):
    dir1 = "/booleanfs2/sahoo/Data/Pradipta/gastric/BN/"
    cfile = dir1 + "figures/gc-net-4-cls.txt"
    fp = open(cfile, "r")
    nhash = {}
    for line in fp:
        line = line.strip();
        ll = line.split("\t");
        nhash[ll[0]] = ll[2:]
    fp.close();
    l1 = []
    wt1 = []
    for i in range(len(v1)):
        k = v1[i]
        if k in nhash:
            l1 += [set(nhash[k])]
            wt1 += [wt[i]]
    return wt1, l1

#https://peerj.com/articles/8873/
def getCao2020(tn = 1):
    if (tn == 2):
        l1 = [ ["CLUL1", "CNGB1", "ROM1", "LRRC39", "RDH12"] ]
        wt1 = [-1]
        return wt1, l1
    l1 = [ ["CLUL1", "LRRC4C", "RDH12", "LRRC39", "TDRD7", "PODXL", "NUSAP1", "ABLIM1",
            "TOP2A", "MCM2", "VSX1", "ROM1", "TCF19", "MCM6", "CNGB1", "NRL",
            "KIFC1", "SORBS2", "MKI67", "RGR", "CHEK1"] ]
    wt1 = [-1]
    return wt1, l1

class RBAnalysis:

    def __init__(self):
        self.db = hu.Database("/booleanfs2/sahoo/Hegemon/explore.conf")
        self.normal = []
        self.uc = []
        self.cd = []
        self.ibd = []
        self.st1 = []
        self.st2 = []
        self.st3 = []
        self.otype = 0

    def printInfo(self):
        print (self.h.rdataset.getName() + " (n = " + str(self.h.getNum()) + ")")
        if len(self.ibd) > 0:
            url = "http://hegemon.ucsd.edu/Tools/explore.php?key=polyps&id="
            if self.dbid.startswith("CRC"):
                url = "http://hegemon.ucsd.edu/Tools/explore.php?key=colon&id="
            print (len(self.order), len(self.normal), \
                    len(self.uc), len(self.cd), self.h.getSource(), \
                    url + self.dbid, self.dbid)
            return
        url = "http://hegemon.ucsd.edu/Tools/explore.php?key=blood:leukemia&id="
        if self.dbid.startswith("PLP"):
            url = "http://hegemon.ucsd.edu/Tools/explore.php?key=polyps&id="
        if self.dbid.startswith("CRC"):
            url = "http://hegemon.ucsd.edu/Tools/explore.php?key=colon&id="
        if self.dbid.startswith("MAC"):
            url = "http://hegemon.ucsd.edu/Tools/explore.php?key=mac&id="
        if self.dbid.startswith("LIV"):
            url = "http://hegemon.ucsd.edu/Tools/explore.php?key=liver&id="
        if self.dbid.startswith("GS"):
            url = "http://hegemon.ucsd.edu/Tools/explore.php?key=gastric&id="
        if self.dbid.startswith("RB") or self.dbid.startswith("RT"):
            url = "http://hegemon.ucsd.edu/Tools/explore.php?key=rb:rt&id="
        print (len(self.order), len(self.st1), \
                len(self.st2), len(self.st3), self.h.getSource(), \
                url + self.dbid, self.dbid)

    def convertMm(self, gene_groups, genes):
        self.gene_groups = getGroupsMm(gene_groups)
        self.genes = getGroupsMm([genes])[0]

    def orderData(self, gene_groups, weight):
        self.col_labels = [self.h.headers[i] for i in self.order]
        #ranks, row_labels, expr = getRanks(gene_groups, h)
        ranks, row_labels, row_ids, row_numhi, expr = getRanks2(gene_groups,
                self.h)
        self.f_ranks = mergeRanks(self.h.aRange(), self.h.start, ranks, weight)
        #i1 = getOrder(self.order, self.h.start, ranks, weight)
        arr = [self.f_ranks[i - self.h.start] for i in self.order]
        i1 = [self.order[i] for i in np.argsort(arr)]
        index = np.array([i - self.h.start for i in i1])
        self.cval = np.array([[self.aval[i] for i in i1]])
        #self.data = np.array(expr)[:,index]
        self.ind_r = np.array(sorted(range(len(row_labels)),
            key=lambda x: (row_numhi[x][0], row_numhi[x][1])))
        row_labels = [row_labels[i] for i in self.ind_r]
        row_ids = [row_ids[i] for i in self.ind_r]
        if len(row_ids) == 0:
            return
        self.data = np.array([expr[i] for i in self.ind_r])[:,index]
        self.ranks = ranks
        self.row_labels = row_labels
        self.row_ids = row_ids
        self.row_numhi = row_numhi
        self.expr = expr
        self.i1 = i1
        self.index = index
        self.otype = 1

    def orderData2(self, gene_groups, weight):
        self.col_labels = [self.h.headers[i] for i in self.order]
        #ranks, row_labels, expr = getRanks(gene_groups, h)
        ranks, row_labels, row_ids, row_numhi, expr = getRanks3(gene_groups,
                self.h, self.order)
        self.f_ranks = mergeRanks2(self.order, ranks, weight)
        i1 = [self.order[i] for i in np.argsort(self.f_ranks)]
        index = np.argsort(self.f_ranks)
        self.cval = np.array([[self.aval[i] for i in i1]])
        #self.data = np.array(expr)[:,index]
        self.ind_r = np.array(sorted(range(len(row_labels)),
            key=lambda x: (row_numhi[x][0], row_numhi[x][1])))
        row_labels = [row_labels[i] for i in self.ind_r]
        row_ids = [row_ids[i] for i in self.ind_r]
        self.data = np.array([expr[i] for i in self.ind_r])[:,index]
        self.ranks = ranks
        self.row_labels = row_labels
        self.row_ids = row_ids
        self.row_numhi = row_numhi
        self.expr = expr
        self.i1 = i1
        self.index = index
        self.otype = 2

    def saveData(self, ofile1, ofile2):
        saveHeatmapData(ofile1, self.row_labels, self.row_numhi,
                self.row_ids, self.index, self.expr)
        saveCData(ofile2, self.h, self.i1, self.f_ranks)

    def readData(self, file1, file2):
        row_labels, row_numhi, row_ids, data = readHeatmapData(file1)
        i1, f_ranks = readCData(file2)
        self.ranks = ranks
        self.row_labels = row_labels
        self.row_ids = row_ids
        self.row_numhi = row_numhi
        self.expr = expr
        self.i1 = i1
        self.f_ranks = f_ranks

    def printHeatmap(self, ofile, genes, params):
        i1 = self.i1
        f_ranks = self.f_ranks
        self.params = {'genes': genes,'atypes': self.atypes,'cval': self.cval}
        self.params.update(params)
        ax = plotHeatmap(ofile, self.data, self.col_labels, 
                self.row_labels, self.params)
        actual = [1 if self.aval[i] >= 1 else 0 for i in i1]
        if "actual" in self.params:
            actual = self.params["actual"]
        thr = hu.getThrData(f_ranks)
        nm = (np.max(f_ranks) - np.min(f_ranks))/15
        predicted = [1 if f_ranks[i - self.h.start] >= thr[0] - nm else 0 for i in i1]
        if "thr" in self.params:
            if self.params["thr"] == 1:
                predicted = [1 if f_ranks[i - self.h.start] >= thr[0] else 0 for i in i1]  
            if self.params["thr"] == 2:
                predicted = [1 if f_ranks[i - self.h.start] >= thr[0] + nm else 0 for i in i1]  
        if "predicted" in self.params:
            predicted = self.params["predicted"]
        data_list = {'x' : predicted}
        df = pd.DataFrame(data_list)
        df['y'] = pd.Series(np.array(actual))
        target_names = ['Normal', 'IBD']
        score=convertScore(predicted)
        print (list(actual))
        print (list(predicted))
        ax1 = printReport(actual, predicted, score, target_names)
        tab = pd.crosstab(df.y > 0, df.x > 0)
        print (tab)
        print (fisher_exact(tab))
        print ('Fisher Exact pvalue =', fisher_exact(tab)[1])
        return ax, ax1

    def printHeatmap2(self, ofile, genes, params):
        i1 = self.i1
        f_ranks = self.f_ranks
        self.params = {'genes': genes,'atypes': self.atypes,'cval': self.cval}
        self.params.update(params)
        ax = plotHeatmap(ofile, self.data, self.col_labels, 
                self.row_labels, self.params)
        actual = [1 if self.aval[i] >= 1 else 0 for i in i1]
        if "actual" in self.params:
            actual = self.params["actual"]
        thr = hu.getThrData([f_ranks[i - self.h.start] for i in i1])
        nm = (np.max(f_ranks) - np.min(f_ranks))/15
        predicted = [1 if f_ranks[i - self.h.start] >= thr[0] - nm else 0 for i in i1]
        if "thr" in self.params:
            if self.params["thr"] == 1:
                predicted = [1 if f_ranks[i - self.h.start] >= thr[0] else 0 for i in i1]  
            if self.params["thr"] == 2:
                predicted = [1 if f_ranks[i - self.h.start] >= thr[0] + nm else 0 for i in i1]  
        if "predicted" in self.params:
            predicted = self.params["predicted"]
        data_list = {'x' : predicted}
        df = pd.DataFrame(data_list)
        df['y'] = pd.Series(np.array(actual))
        target_names = ['Normal', 'IBD']
        score=convertScore(predicted)
        print (list(actual))
        print (list(predicted))
        ax1 = printReport(actual, predicted, score, target_names)
        tab = pd.crosstab(df.y > 0, df.x > 0)
        print (tab)
        print (fisher_exact(tab))
        print ('Fisher Exact pvalue =', fisher_exact(tab)[1])
        return ax, ax1

    def printHeatmap3(self, ofile, genes, params):
        i1 = self.i1
        f_ranks = self.f_ranks
        self.params = {'genes': genes,'atypes': self.atypes,'cval': self.cval}
        self.params.update(params)
        ax = plotHeatmap(ofile, self.data, self.col_labels, 
                self.row_labels, self.params)
        actual = [1 if self.aval[i] >= 1 else 0 for i in i1]
        if "actual" in self.params:
            actual = self.params["actual"]
        thr = hu.getThrData(f_ranks)
        nm = (np.max(f_ranks) - np.min(f_ranks))/15
        i2 = np.argsort(f_ranks)
        predicted = [1 if f_ranks[i] >= thr[0] - nm else 0 for i in i2]
        if "thr" in self.params:
            if self.params["thr"] == 1:
                predicted = [1 if f_ranks[i] >= thr[0] else 0 for i in i2]  
            if self.params["thr"] == 2:
                predicted = [1 if f_ranks[i] >= thr[0] + nm else 0 for i in i2]  
        if "predicted" in self.params:
            predicted = self.params["predicted"]
        data_list = {'x' : predicted}
        df = pd.DataFrame(data_list)
        df['y'] = pd.Series(np.array(actual))
        target_names = ['Normal', 'IBD']
        score=convertScore(predicted)
        print (list(actual))
        print (list(predicted))
        ax1 = printReport(actual, predicted, score, target_names)
        tab = pd.crosstab(df.y > 0, df.x > 0)
        print (tab)
        print (fisher_exact(tab))
        print ('Fisher Exact pvalue =', fisher_exact(tab)[1])
        return ax, ax1

    def printTitleBar(self, params):
        self.params = {'atypes': self.atypes,'cval': self.cval}
        self.params.update(params)
        ax = plotTitleBar(self.params['cval'], \
                self.params['atypes'], self.params)
        ax.text(len(self.params['cval'][0]), 0, self.h.getSource())
        return ax

    def normMacrophageGene(self, gene, params):
        exp1 = self.h.getExprData(gene)
        thr1 = self.h.getThrData(gene)
        if self.otype == 2:
            tdata = [[float(exp1[self.order[i]]), float(self.f_ranks[i])] \
                    for i in range(len(self.order))]
        else:
            tdata = [[float(exp1[i]), self.f_ranks[i - self.h.start]] \
                    for i in self.h.aRange()]

        data = pd.DataFrame(tdata, columns=["x", "y"])
        from sklearn.linear_model import LinearRegression
        linreg = LinearRegression(normalize=True)
        if "select" in params and params["select"] == 2:
            return
        if "select" in params and params["select"] == 1 and self.otype != 2:
            tdata = [[float(exp1[i]), self.f_ranks[i - self.h.start]] \
                    for i in self.order]
            df = pd.DataFrame(tdata, columns=["x", "y"])
            linreg.fit(df['x'].reshape(-1, 1),df['y'])
            y_pred = linreg.predict(data['x'].reshape(-1, 1))
        else:
            linreg.fit(data['x'].reshape(-1, 1),data['y'])
            y_pred = linreg.predict(data['x'].reshape(-1, 1))
        thr = 0
        if "thr" in params:
            if params["thr"] == 0:
                thr = thr1[2]
            if params["thr"] == 1:
                thr = thr1[0]
            if params["thr"] == 2:
                thr = thr1[3]
            if params["thr"] == 3:
                thr = 0
            if params["thr"] == 4 and "tval" in params:
                thr = params["tval"]
        def getCol(data, y_pred, i, thr):
            if data['x'][i] <= thr:
                return 'grey'
            if data['y'][i] >= y_pred[i]:
                return 'green'
            return 'red'
        if self.otype == 2:
            col = [getCol(data, y_pred, i, thr) for i in range(len(self.order))]
        else:
            col = [getCol(data, y_pred, i - self.h.start, thr) for i in self.h.aRange()]
            col = []
            for i in self.h.aRange():
                if i in self.order:
                    col.extend([getCol(data, y_pred, i - self.h.start, thr)])
                else:
                    col.extend(["#CCCCCC"])
        ax = data.plot.scatter(x='x', y='y', c=col)
        ax.plot(data['x'], y_pred, c='Orange')
        ax.set_xlabel(gene)
        ax.set_ylabel("Gene Signature")
        df = pd.DataFrame()
        df['x'] = data['x']
        df['y'] = data['y']
        df['y1'] = (data['y'] - y_pred)
        ax = df.plot.scatter(x='x', y='y1', c=col)
        ax.set_xlabel(gene)
        ax.set_ylabel("Gene Signature")
        if self.otype == 2:
            i3 = [i for i in range(len(self.order)) if df['x'][i] > thr]
            self.f_ranks = [df['y1'][i] for i in i3]
            self.order = [self.order[i] for i in i3]
            self.i1 = [self.order[i] for i in np.argsort(self.f_ranks)]
            index = [i3[i] for i in np.argsort(self.f_ranks)]
        else:
            self.f_ranks = df['y1']
            self.order = [i for i in self.order if df['x'][i - self.h.start] > thr]
            arr = [self.f_ranks[i - self.h.start] for i in self.order]
            self.i1 = [self.order[i] for i in np.argsort(arr)]
            index = np.array([i - self.h.start for i in self.i1])
        self.cval = np.array([[self.aval[i] for i in self.i1]])
        self.data = np.array([self.expr[i] for i in self.ind_r])[:,index]

    def normMacrophage(self, params):
        return self.normMacrophageGene("TYROBP", params)

    def getMetrics(ana, actual = None, ahash = None):
        if actual is None:
            actual = [ana.aval[i] for i in ana.i1]
        if ana.otype == 2:
            score = [ana.f_ranks[i] for i in ana.i1]
        else:
            if ahash is None:
                score = [ana.f_ranks[i - ana.h.start] for i in ana.i1]
            else:
                score = [ana.f_ranks[ana.i1[i] - ana.h.start] \
                        for i in range(len(ana.i1)) \
                        if ana.cval[0][i] in  ahash ]
        res = None
        fpr, tpr, thrs = roc_curve(actual, score, pos_label=1)
        roc_auc = auc(fpr, tpr)
        res = "%.2f" % roc_auc
        return res

    def getMetrics2(ana, actual, ahash = None, fthr = None):
        if ana.otype == 2:
            score = [ana.f_ranks[i] for i in ana.i1]
        else:
            if ahash is None:
                score = [ana.f_ranks[i - ana.h.start] for i in ana.i1]
            else:
                score = [ana.f_ranks[ana.i1[i] - ana.h.start] \
                        for i in range(len(ana.i1)) \
                        if ana.cval[0][i] in  ahash ]
        res = [None, None, None]
        for i in range(3):
            fpr, tpr, thrs = roc_curve(actual, score, pos_label=i)
            roc_auc = auc(fpr, tpr)
            if roc_auc < 0.5:
                roc_auc = 1 - roc_auc
            res[i] = "%.2f" % roc_auc
        thr = hu.getThrData(score[:-2])
        nm = (np.max(score) - np.min(score))/15
        if fthr is None:
            fthr = thr[0]
        if fthr == "thr0":
            fthr = thr[0] - nm
        if fthr == "thr2":
            fthr = thr[0] + nm
        if fthr == "thr3":
            fthr = thr[0] + 3 * nm
        if fthr < np.min(score) or fthr > np.max(score):
            fthr = thr[0]
        predicted = [1 if ana.f_ranks[i - ana.h.start] >= fthr else 0 for i in ana.i1]
        print (thr)
        print (nm, fthr)
        print (score)
        print (actual)
        print (predicted)
        x0 = [i for i in predicted if i == 0]
        x1 = [i for i in predicted if i == 1]
        if len(x0) == 0:
            predicted[0] = 0
        if len(x1) == 0:
            predicted[0] = 1
        res[1] = "%.2f" % accuracy_score(actual, predicted)
        data_list = {'x' : predicted}
        df = pd.DataFrame(data_list)
        df['y'] = pd.Series(np.array(actual))
        tab = pd.crosstab(df.y > 0, df.x > 0)
        res[2] = "%.3g" % fisher_exact(tab)[1]
        return res

    def densityPlot(self, ax=None, acolor=acolor):
        x = [self.aval[i] for i in self.i1]
        df = pd.Series(x)
        for i in range(len(self.atypes)):
            df1 = pd.DataFrame(pd.Series(df[df == i].index),
                    columns=[self.atypes[i]])
            if len(df1[self.atypes[i]]) <= 1:
                continue
            if ax is None:
                ax = df1.plot.kde(bw_method=1.0, c=acolor[i])
            else:
                ax = df1.plot.kde(bw_method=1.0, ax = ax, c=acolor[i])

        ax.set_title("Density plot")
        ax.set_xlabel("Sample rank")
        return ax

    def getSurvival(self, dbid = "CRC35.3"):
        self.db = hu.Database("/booleanfs2/sahoo/Hegemon/explore.conf")
        self.dbid = dbid
        self.h = hu.Hegemon(self.db.getDataset(self.dbid))
        self.h.init()
        self.h.initPlatform()
        self.h.initSurv()
        atype = self.h.getSurvName("status")
        atypes = ['Censor', 'Relapse']
        ahash = {"0": 0, "1":1}
        aval = [ahash[atype[i]] if atype[i] in ahash \
                        else None for i in range(self.h.getNum()+2)]
        st0 = [ i for i in self.h.aRange() if aval[i] == 0]
        st1 = [ i for i in self.h.aRange() if aval[i] == 1]
        self.aval = aval
        self.atype = atype
        self.atypes = atypes
        self.order = st0 + st1
        self.printInfo()

    def printSurvival(self, fthr = None, pG = None, genex = "CDX2",
            ct = None, ax = None):
        f_ranks = self.f_ranks
        order = self.order
        thr = hu.getThrData(f_ranks)
        nm = (np.max(f_ranks) - np.min(f_ranks))/16
        if fthr is None:
            fthr = thr[0]
        if fthr == "thr0":
            fthr = thr[0] - nm
        if fthr == "thr2":
            fthr = thr[0] + nm
        if fthr == "thr3":
            fthr = thr[0] + 3 * nm
        print (thr)
        print (nm, fthr)
        g1 = [i for i in order if f_ranks[i - self.h.start] < fthr]
        g2 = [i for i in order if f_ranks[i - self.h.start] >= fthr]
        if pG is None:
            pG = [ ["Low", "green", g1], ["High", "red", g2]]
        obj = hu.getHegemonPatientData(self.dbid, 'time')
        time = obj[1]
        obj = hu.getHegemonPatientData(self.dbid, 'status')
        status = obj[1]
        if ct is not None:
            time, status = hu.censor(time, status, ct)
        sax = hu.survival(time, status, pG, ax)
        ax = None
        if genex is not None:
            df = pd.DataFrame()
            df["f_ranks"] = pd.to_numeric(pd.Series(f_ranks))
            e = self.h.getExprData(genex)
            df[genex] = pd.to_numeric(pd.Series(e[2:]))
            ax = df.plot.scatter(x=genex, y='f_ranks')
        return sax, ax

    def printCoxSurvival(ana, fthr = None):
        score = [ana.f_ranks[i - ana.h.start] for i in ana.h.aRange()]
        thr = hu.getThrData(score)
        nm = (np.max(ana.f_ranks) - np.min(ana.f_ranks))/16
        if fthr is None:
            fthr = thr[0]
        if fthr == "thr0":
            fthr = thr[0] - nm
        if fthr == "thr2":
            fthr = thr[0] + nm
        if fthr == "thr3":
            fthr = thr[0] + 3 * nm
        print (thr)
        print (nm, fthr)
        c1 = ["", ""] + [1 if ana.f_ranks[i - ana.h.start] >= fthr else 0 for i in
                ana.h.aRange()]
        obj = hu.getHegemonPatientData(ana.dbid, 'time')
        time = obj[1]
        obj = hu.getHegemonPatientData(ana.dbid, 'status')
        status = obj[1]
        mdata = {"time": time, "status": status, "c1" : c1}
        df = pd.DataFrame(mdata)
        df.drop([0, 1], inplace=True)
        df.replace('', np.nan, inplace=True)
        df.dropna(inplace=True)

        import rpy2.robjects as ro
        ro.r('library(survival)')
        ro.r("time <- c(" + ",".join(df['time']) + ")")
        ro.r("status <- c(" + ",".join(df['status']) + ")")
        ro.r("c1 <- c(" + ",".join([str(i) for i in df['c1']]) + ")")
        ro.r('x <- coxph(Surv(time, status) ~ c1)')
        ro.r('s <- summary(x)')
        print (ro.r('s'))
        t = [float(i) for i in df['time']]
        s = [int(i) for i in df['status']]
        g = [int(i) for i in df['c1']]
        from lifelines.statistics import multivariate_logrank_test
        from matplotlib.legend import Legend
        res = multivariate_logrank_test(t, g, s)
        print ("p = %.2g" % res.p_value)

    def getJSTOM(self):
        self.getSurvival("CRC35.3")

    def getBos(self):
        self.getSurvival("BC20")

    def getSailaja2019(self, tn=1):
        self.db = hu.Database("/booleanfs2/sahoo/Hegemon/explore.conf")
        self.dbid = "RB5"
        self.h = hu.Hegemon(self.db.getDataset(self.dbid))
        self.h.init()
        self.h.initPlatform()
        self.h.initSurv()
        atype = self.h.getSurvName('c Type');
        atypes = ['N', 'T']
        ahash = {'RB':1, 'Control Retina':0}
        if tn == 2:
            l1 = [i for i in self.h.aRange() if self.h.headers[i] == "Retina_5_years"]
            atype[l1[0]] = None;
        for i in range(len(atypes)):
            ahash[atypes[i]] = i
        aval = [ahash[i] if i in ahash else None for i in atype]
        expg = [i for i in self.h.aRange() if aval[i] is not None]
        self.st1 = [i for i in self.h.aRange() if aval[i] == 0]
        self.st2 = [i for i in self.h.aRange() if aval[i] == 1]
        self.st3 = [i for i in self.h.aRange() if aval[i] == 2]
        self.aval = aval
        self.atype = atype
        self.atypes = atypes
        self.order = expg
        self.printInfo()

    def getRavi2017(self):
        self.db = hu.Database("/booleanfs2/sahoo/Hegemon/explore.conf")
        self.dbid = "RB7"
        self.h = hu.Hegemon(self.db.getDataset(self.dbid))
        self.h.init()
        self.h.initPlatform()
        self.h.initSurv()
        atype = self.h.getSurvName('c Type');
        atypes = ['N', 'T']
        ahash = {'Invasive RB':1, 'Non-invasive RB':1, 'Normal Retina':0}
        for i in range(len(atypes)):
            ahash[atypes[i]] = i
        aval = [ahash[i] if i in ahash else None for i in atype]
        expg = [i for i in self.h.aRange() if aval[i] is not None]
        self.st1 = [i for i in self.h.aRange() if aval[i] == 0]
        self.st2 = [i for i in self.h.aRange() if aval[i] == 1]
        self.st3 = [i for i in self.h.aRange() if aval[i] == 2]
        self.aval = aval
        self.atype = atype
        self.atypes = atypes
        self.order = expg
        self.printInfo()

    def getSailaja2020(self, tn=1):
        self.db = hu.Database("/booleanfs2/sahoo/Hegemon/explore.conf")
        self.dbid = "RB13"
        self.h = hu.Hegemon(self.db.getDataset(self.dbid))
        self.h.init()
        self.h.initPlatform()
        self.h.initSurv()
        atype = self.h.getSurvName('c Type');
        atypes = ['RT', 'RB', 'Y79', 'Weri', 'NCC']
        ahash = {'RB':1, 'Control Retina':0, 'Plate':4, 'GF':4, 'Xeno':4,
                'Y79-WT':2, 'Y79-CANT1':2, 'Weri-WT':3, 'Weri-CANT1':3}
        if tn == 2:
            atypes = ['RT', 'RB']
            ahash = {'RB':1, 'Control Retina':0, 'Plate':1, 'GF':1, 'Xeno':1,
                    'Y79-WT':1, 'Y79-CANT1':1, 'Weri-WT':1, 'Weri-CANT1':1}
        if tn == 3:
            atypes = ['RT', 'RB', 'Y79', 'Weri', 'NCC']
            ahash = {'RB':1, 'Control Retina':0, 'Xeno':4, 'Y79-WT':2, 'Weri-WT':3}
        if tn == 4:
            atypes = ['RT', 'NCC', 'CL']
            ahash = {'Control Retina':0, 'Xeno':1, 'Y79-WT':2, 'Weri-WT':2}
            atype = [atype[i] if self.h.headers[i] != 'Retina_5_years' else
                    None for i in range(len(atype))]
        for i in range(len(atypes)):
            ahash[atypes[i]] = i
        aval = [ahash[i] if i in ahash else None for i in atype]
        expg = [i for i in self.h.aRange() if aval[i] is not None]
        self.st1 = [i for i in self.h.aRange() if aval[i] == 0]
        self.st2 = [i for i in self.h.aRange() if aval[i] == 1]
        self.st3 = [i for i in self.h.aRange() if aval[i] == 2]
        self.aval = aval
        self.atype = atype
        self.atypes = atypes
        self.order = expg
        self.printInfo()

    def getKooi2015(self, tn=1):
        self.db = hu.Database("/booleanfs2/sahoo/Hegemon/explore.conf")
        self.dbid = "RB1"
        self.h = hu.Hegemon(self.db.getDataset(self.dbid))
        self.h.init()
        self.h.initPlatform()
        self.h.initSurv()
        atype = self.h.getSurvName('c UHC');
        atypes = ['2', '1', '3']
        ahash = {'group 2':0, 'group 1':1, 'group 3':2}
        if (tn == 2):
            atypes = ['2', '1']
            ahash = {'group 2':0, 'group 1':1}
        if (tn == 3):
            atype = self.h.getSurvName('c differentiation');
            atypes = ['w', 'p']
            ahash = {'intermediate':0, 'well':0, 'poor':1}
        if (tn == 4):
            atype = self.h.getSurvName('c laterality');
            atypes = ['b', 'u']
            ahash = {'unilateral':1, 'bilateral':0}
        for i in range(len(atypes)):
            ahash[atypes[i]] = i
        aval = [ahash[i] if i in ahash else None for i in atype]
        expg = [i for i in self.h.aRange() if aval[i] is not None]
        self.st1 = [i for i in self.h.aRange() if aval[i] == 0]
        self.st2 = [i for i in self.h.aRange() if aval[i] == 1]
        self.st3 = [i for i in self.h.aRange() if aval[i] == 2]
        self.aval = aval
        self.atype = atype
        self.atypes = atypes
        self.order = expg
        self.printInfo()

    def getGratias2007(self, tn=1):
        self.db = hu.Database("/booleanfs2/sahoo/Hegemon/explore.conf")
        self.dbid = "RB2"
        self.h = hu.Hegemon(self.db.getDataset(self.dbid))
        self.h.init()
        self.h.initPlatform()
        self.h.initSurv()
        atype = self.h.getSurvName('c Type');
        atypes = ['b', 'u']
        ahash = {'sporadic unilateral Rb':1, 'sporadic bilateral Rb':0}
        for i in range(len(atypes)):
            ahash[atypes[i]] = i
        aval = [ahash[i] if i in ahash else None for i in atype]
        expg = [i for i in self.h.aRange() if aval[i] is not None]
        self.st1 = [i for i in self.h.aRange() if aval[i] == 0]
        self.st2 = [i for i in self.h.aRange() if aval[i] == 1]
        self.st3 = [i for i in self.h.aRange() if aval[i] == 2]
        self.aval = aval
        self.atype = atype
        self.atypes = atypes
        self.order = expg
        self.printInfo()

    def getMcEvoy2011(self, tn=1):
        self.db = hu.Database("/booleanfs2/sahoo/Hegemon/explore.conf")
        self.dbid = "RB3"
        self.h = hu.Hegemon(self.db.getDataset(self.dbid))
        self.h.init()
        self.h.initPlatform()
        self.h.initSurv()
        atype = self.h.getSurvName('c cell type');
        atypes = ['T', 'X', 'C']
        ahash = {'primary tumor':0, 'xenograft-passaged':1,
                'cell line Weril':2, 'cell line RB355':2,
                'cell line RB1 13':2, 'cell line Y79':2}
        for i in range(len(atypes)):
            ahash[atypes[i]] = i
        aval = [ahash[i] if i in ahash else None for i in atype]
        expg = [i for i in self.h.aRange() if aval[i] is not None]
        self.st1 = [i for i in self.h.aRange() if aval[i] == 0]
        self.st2 = [i for i in self.h.aRange() if aval[i] == 1]
        self.st3 = [i for i in self.h.aRange() if aval[i] == 2]
        self.aval = aval
        self.atype = atype
        self.atypes = atypes
        self.order = expg
        self.printInfo()

    def getMcEvoy2011Mm(self, tn=1):
        self.db = hu.Database("/booleanfs2/sahoo/Hegemon/explore.conf")
        self.dbid = "RB4"
        self.h = hu.Hegemon(self.db.getDataset(self.dbid))
        self.h.init()
        self.h.initPlatform()
        self.h.initSurv()
        atype = self.h.getSurvName('c genotype/variation');
        atypes = ['P', 'X', 'S', '7D', '3D', 'T']
        ahash = {'TKO':0, 'MDMX':1, 'p107_single':2,
                'p107_DKO':3, 'p130_DKO':4, 'p130_TKO':5}
        if (tn == 2):
            atypes = ['S', 'D', 'T']
            ahash = {'p107_single':0, 'p107_DKO':1, 'p130_DKO':1, 'p130_TKO':2}
        if (tn == 3):
            atypes = ['SD', 'T']
            ahash = {'p107_single':0, 'p107_DKO':0, 'p130_DKO':0, 'p130_TKO':1}
        for i in range(len(atypes)):
            ahash[atypes[i]] = i
        aval = [ahash[i] if i in ahash else None for i in atype]
        expg = [i for i in self.h.aRange() if aval[i] is not None]
        self.st1 = [i for i in self.h.aRange() if aval[i] == 0]
        self.st2 = [i for i in self.h.aRange() if aval[i] == 1]
        self.st3 = [i for i in self.h.aRange() if aval[i] == 2]
        self.aval = aval
        self.atype = atype
        self.atypes = atypes
        self.order = expg
        self.printInfo()

    def getAldiri2017(self, tn=1):
        self.db = hu.Database("/booleanfs2/sahoo/Hegemon/explore.conf")
        self.dbid = "RB9"
        self.h = hu.Hegemon(self.db.getDataset(self.dbid))
        self.h.init()
        self.h.initPlatform()
        self.h.initSurv()
        atype = self.h.getSurvName('c Type');
        atypes = ['RT', 'RB', 'X']
        ahash = {'Retina':0, 'retinoblastoma diagnosis':1,
                'retinoblastoma xenograft':2}
        if (tn == 2):
            atypes = ['RT', 'RB']
            ahash = {'Retina':0, 'retinoblastoma diagnosis':1,
                    'retinoblastoma xenograft':1}
        for i in range(len(atypes)):
            ahash[atypes[i]] = i
        aval = [ahash[i] if i in ahash else None for i in atype]
        expg = [i for i in self.h.aRange() if aval[i] is not None]
        self.st1 = [i for i in self.h.aRange() if aval[i] == 0]
        self.st2 = [i for i in self.h.aRange() if aval[i] == 1]
        self.st3 = [i for i in self.h.aRange() if aval[i] == 2]
        self.aval = aval
        self.atype = atype
        self.atypes = atypes
        self.order = expg
        self.printInfo()

    def getHudson2018(self, tn=1):
        self.db = hu.Database("/booleanfs2/sahoo/Hegemon/explore.conf")
        self.dbid = "RB11"
        self.h = hu.Hegemon(self.db.getDataset(self.dbid))
        self.h.init()
        self.h.initPlatform()
        self.h.initSurv()
        atype = self.h.getSurvName('c src1');
        atypes = ['RT', 'RB', 'RC']
        ahash = {'Retinoblastoma':1,
                'Matched normal retina':0,
                'Matched retinocytoma':2}
        if tn == 2:
            atypes = ['RT', 'RB']
            ahash = {'Retinoblastoma':1, 'Matched normal retina':0}
        if tn == 3:
            atype = self.h.getSurvName('c anaplasia');
            atypes = ['C', 'L', 'M', 'S']
            ahash = {'Mild':1, 'Moderate':2, 'Severe':3, '':0}
        if tn == 4:
            atype = self.h.getSurvName('c anaplasia');
            atypes = ['C', 'S']
            ahash = {'Mild':0, 'Moderate':0, 'Severe':1, '':0}
        if tn == 5:
            atype = self.h.getSurvName('c Distant metastasis');
            atypes = ['C', 'M']
            ahash = {'No':0, 'Brain':1, 'Bone':1, '"Bone, Liver"':1,
                    '"Liver, Bone marrow"':1}
        if tn == 6:
            atype = self.h.getSurvName('c HRPF');
            atypes = ['C', 'I']
            ahash = {'No':0, 'Postlaminar ON':1,
                    '"Postlaminar ON, massive choroidal invasion"':1,
                    'Massive choroidal invasion':1,
                    '"Laminar ON, nonmassive choroidal invasion"':1}
        for i in range(len(atypes)):
            ahash[atypes[i]] = i
        aval = [ahash[i] if i in ahash else None for i in atype]
        expg = [i for i in self.h.aRange() if aval[i] is not None]
        self.st1 = [i for i in self.h.aRange() if aval[i] == 0]
        self.st2 = [i for i in self.h.aRange() if aval[i] == 1]
        self.st3 = [i for i in self.h.aRange() if aval[i] == 2]
        self.aval = aval
        self.atype = atype
        self.atypes = atypes
        self.order = expg
        self.printInfo()

    def getAldiri2015(self, tn=1):
        self.db = hu.Database("/booleanfs2/sahoo/Hegemon/explore.conf")
        self.dbid = "RB12"
        self.h = hu.Hegemon(self.db.getDataset(self.dbid))
        self.h.init()
        self.h.initPlatform()
        self.h.initSurv()
        atype = self.h.getSurvName('c genotype/variation');
        atypes = ['W', 'M']
        ahash = {'wt':0, 'Brg -/-':1}
        if (tn == 2):
            atype = self.h.getSurvName('c age');
            atypes = ['E14.5', 'P0', 'P12', 'P21']
            ahash = {}
        if (tn == 3):
            atype = self.h.getSurvName('c age');
            atypes = ['P21', 'P12']
            ahash = {}
        for i in range(len(atypes)):
            ahash[atypes[i]] = i
        aval = [ahash[i] if i in ahash else None for i in atype]
        expg = [i for i in self.h.aRange() if aval[i] is not None]
        self.st1 = [i for i in self.h.aRange() if aval[i] == 0]
        self.st2 = [i for i in self.h.aRange() if aval[i] == 1]
        self.st3 = [i for i in self.h.aRange() if aval[i] == 2]
        self.aval = aval
        self.atype = atype
        self.atypes = atypes
        self.order = expg
        self.printInfo()

    def getKooi2016(self, tn=1):
        self.db = hu.Database("/booleanfs2/sahoo/Hegemon/explore.conf")
        self.dbid = "RB14"
        self.h = hu.Hegemon(self.db.getDataset(self.dbid))
        self.h.init()
        self.h.initPlatform()
        self.h.initSurv()
        atype = self.h.getSurvName('c cell line');
        atypes = ['M', 'S'] # based on chr 17 gain
        ahash = {'UMC-RB-26':0, 'RB383':0, 'RB247':1,
                'RB176':1, 'RB381':0, 'RB1':1, 'Y79':1}
        for i in range(len(atypes)):
            ahash[atypes[i]] = i
        aval = [ahash[i] if i in ahash else None for i in atype]
        expg = [i for i in self.h.aRange() if aval[i] is not None]
        self.st1 = [i for i in self.h.aRange() if aval[i] == 0]
        self.st2 = [i for i in self.h.aRange() if aval[i] == 1]
        self.st3 = [i for i in self.h.aRange() if aval[i] == 2]
        self.aval = aval
        self.atype = atype
        self.atypes = atypes
        self.order = expg
        self.printInfo()

    def getVenkatesan2012(self, tn=1):
        self.db = hu.Database("/booleanfs2/sahoo/Hegemon/explore.conf")
        self.dbid = "RB16"
        self.h = hu.Hegemon(self.db.getDataset(self.dbid))
        self.h.init()
        self.h.initPlatform()
        self.h.initSurv()
        atype = self.h.getSurvName('c src1');
        atypes = ['C', 'M']
        ahash = {'Anti-HMGA2 siRNA treated Y79 cells':1, 'Untreated y79 cells':0}
        for i in range(len(atypes)):
            ahash[atypes[i]] = i
        aval = [ahash[i] if i in ahash else None for i in atype]
        expg = [i for i in self.h.aRange() if aval[i] is not None]
        self.st1 = [i for i in self.h.aRange() if aval[i] == 0]
        self.st2 = [i for i in self.h.aRange() if aval[i] == 1]
        self.st3 = [i for i in self.h.aRange() if aval[i] == 2]
        self.aval = aval
        self.atype = atype
        self.atypes = atypes
        self.order = expg
        self.printInfo()

    def getNalini2013(self, tn=1):
        self.db = hu.Database("/booleanfs2/sahoo/Hegemon/explore.conf")
        self.dbid = "RB18"
        self.h = hu.Hegemon(self.db.getDataset(self.dbid))
        self.h.init()
        self.h.initPlatform()
        self.h.initSurv()
        atype = self.h.getSurvName('c src1');
        atypes = ['RT', 'RBn', 'RBc']
        ahash = {'Retinoblastoma -NO Chemotreated':1,
                'Retinoblastoma -Chemotreated':2,
                'Retina from healthy adult':0}
        if (tn == 2):
            atypes = ['RT', 'RB']
            ahash = {'Retinoblastoma -NO Chemotreated':1,
                    'Retinoblastoma -Chemotreated':1,
                    'Retina from healthy adult':0}
        if (tn == 3):
            atypes = ['RBc', 'RBn']
            ahash = {'Retinoblastoma -NO Chemotreated':1,
                    'Retinoblastoma -Chemotreated':0}
        for i in range(len(atypes)):
            ahash[atypes[i]] = i
        aval = [ahash[i] if i in ahash else None for i in atype]
        expg = [i for i in self.h.aRange() if aval[i] is not None]
        self.st1 = [i for i in self.h.aRange() if aval[i] == 0]
        self.st2 = [i for i in self.h.aRange() if aval[i] == 1]
        self.st3 = [i for i in self.h.aRange() if aval[i] == 2]
        self.aval = aval
        self.atype = atype
        self.atypes = atypes
        self.order = expg
        self.printInfo()

    def getMcCurdy2017(self, tn=1):
        self.db = hu.Database("/booleanfs2/sahoo/Hegemon/explore.conf")
        self.dbid = "RB19"
        self.h = hu.Hegemon(self.db.getDataset(self.dbid))
        self.h.init()
        self.h.initPlatform()
        self.h.initSurv()
        atype = self.h.getSurvName('c genotype');
        atypes = ['Wt', 'RB', 'DKO']
        ahash = {'Rb KO':1, 'wildtype':0, 'Rb/p107 DKO':2}
        if (tn == 2):
            atypes = ['Wt', 'Mut']
            ahash = {'Rb KO':1, 'wildtype':0, 'Rb/p107 DKO':1}
        if (tn == 3):
            atypes = ['Rb', 'DKO']
            ahash = {'Rb KO':0, 'Rb/p107 DKO':1}
        for i in range(len(atypes)):
            ahash[atypes[i]] = i
        aval = [ahash[i] if i in ahash else None for i in atype]
        expg = [i for i in self.h.aRange() if aval[i] is not None]
        self.st1 = [i for i in self.h.aRange() if aval[i] == 0]
        self.st2 = [i for i in self.h.aRange() if aval[i] == 1]
        self.st3 = [i for i in self.h.aRange() if aval[i] == 2]
        self.aval = aval
        self.atype = atype
        self.atypes = atypes
        self.order = expg
        self.printInfo()

    def getChai2018(self, tn=1):
        self.db = hu.Database("/booleanfs2/sahoo/Hegemon/explore.conf")
        self.dbid = "RB20"
        self.h = hu.Hegemon(self.db.getDataset(self.dbid))
        self.h.init()
        self.h.initPlatform()
        self.h.initSurv()
        atype = self.h.getSurvName('c tissue');
        atypes = ['RT', 'RB']
        ahash = {'normal tissue adjacent to tumor':0, 'retinoblastoma tumor':1}
        for i in range(len(atypes)):
            ahash[atypes[i]] = i
        aval = [ahash[i] if i in ahash else None for i in atype]
        expg = [i for i in self.h.aRange() if aval[i] is not None]
        self.st1 = [i for i in self.h.aRange() if aval[i] == 0]
        self.st2 = [i for i in self.h.aRange() if aval[i] == 1]
        self.st3 = [i for i in self.h.aRange() if aval[i] == 2]
        self.aval = aval
        self.atype = atype
        self.atypes = atypes
        self.order = expg
        self.printInfo()

    def getSaengwimol2018(self, tn=1):
        self.db = hu.Database("/booleanfs2/sahoo/Hegemon/explore.conf")
        self.dbid = "RB21"
        self.h = hu.Hegemon(self.db.getDataset(self.dbid))
        self.h.init()
        self.h.initPlatform()
        self.h.initSurv()
        atype = self.h.getSurvName('c src1');
        atypes = ['RT', 'RB']
        ahash = {'Culture of retinal tumor':1, 'Retinal tumor':1, 'retina':0,
                'Retina':0}
        if (tn == 2):
            atypes = ['RT', 'RB']
            ahash = {'Retinal tumor':1, 'retina':0, 'Retina':0}
        for i in range(len(atypes)):
            ahash[atypes[i]] = i
        aval = [ahash[i] if i in ahash else None for i in atype]
        expg = [i for i in self.h.aRange() if aval[i] is not None]
        self.st1 = [i for i in self.h.aRange() if aval[i] == 0]
        self.st2 = [i for i in self.h.aRange() if aval[i] == 1]
        self.st3 = [i for i in self.h.aRange() if aval[i] == 2]
        self.aval = aval
        self.atype = atype
        self.atypes = atypes
        self.order = expg
        self.printInfo()

    def getKalmodia2017(self, tn=1):
        self.db = hu.Database("/booleanfs2/sahoo/Hegemon/explore.conf")
        self.dbid = "RB22"
        self.h = hu.Hegemon(self.db.getDataset(self.dbid))
        self.h.init()
        self.h.initPlatform()
        self.h.initSurv()
        atype = self.h.getSurvName('c Title');
        atype = [re.sub("_.*", "", str(k)) for k in atype]
        atypes = ['Y79', 'GNP', 'GNP-HDM2']
        ahash = {'GNPs':1, 'Control':0, 'GNPs-HDM2':2}
        for i in range(len(atypes)):
            ahash[atypes[i]] = i
        aval = [ahash[i] if i in ahash else None for i in atype]
        expg = [i for i in self.h.aRange() if aval[i] is not None]
        self.st1 = [i for i in self.h.aRange() if aval[i] == 0]
        self.st2 = [i for i in self.h.aRange() if aval[i] == 1]
        self.st3 = [i for i in self.h.aRange() if aval[i] == 2]
        self.aval = aval
        self.atype = atype
        self.atypes = atypes
        self.order = expg
        self.printInfo()

