library(DESeq2)
library(tximport)
library(tximportData)
dir <- "/booleanfs2/sahoo/Data/RB"
dir2 <- c("project-2", "project-2", "project-2", "project-1", "project-1",
          "Human/GSE141327", "Human/GSE141327")
sample1 <- c("Xeno_2_RNA","Xeno_3","Xeno_1", "172_22", "348_12",
             "GSM4200864", "GSM4200866")
files <- file.path(dir, dir2, "processed", sample1, "kallisto.out", "abundance.h5")
names(files) <- paste0("sample", 1:7)

file1 = "/booleanfs2/sahoo/Data/SeqData/genome/Homo_sapiens.GRCh38.94.chr_patch_hapl_scaff.tv.txt"
library(readr)
tx2gene <- read_tsv(file1, col_names = FALSE)
tx2gene <- tx2gene[,1:2]
colnames(tx2gene) <- c("TXNAME", "GENEID")
head(tx2gene)

txi.kallisto <- tximport(files, type = "kallisto", tx2gene = tx2gene)
head(txi.kallisto$counts)
sampleTable <- data.frame(condition = 
                          factor(c(rep('NCC', 3), rep('Normal', 2), rep('Cl', 2))))
rownames(sampleTable) <- colnames(txi.kallisto$counts)

dds <- DESeqDataSetFromTximport(txi.kallisto, sampleTable, ~condition)

dds <- DESeq(dds)
res <- results(dds, contrast=c("condition", "Normal", "Cl"), cooksCutoff=FALSE)
res <- subset(res, padj < 0.2)
summary(res)
sigfile <- "figures/diff-1.txt"
data <- as.data.frame(res[ order(res$log2FoldChange), ])
write.csv(data, file = sigfile)

res <- results(dds, contrast=c("condition", "Normal", "NCC"), cooksCutoff=FALSE)
res <- subset(res, padj < 0.2)
summary(res)
sigfile <- "figures/diff-2.txt"
data <- as.data.frame(res[ order(res$log2FoldChange), ])
write.csv(data, file = sigfile)

res <- results(dds, contrast=c("condition", "Cl", "NCC"), cooksCutoff=FALSE)
res <- subset(res, padj < 0.2)
summary(res)
sigfile <- "figures/diff-3.txt"
data <- as.data.frame(res[ order(res$log2FoldChange), ])
write.csv(data, file = sigfile)


